package fbg.jdbc.client;

// Title:       JDBC 2.0 Driver for FBG Data Server
// Version:     1.0
// Copyright:   February 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.io.*;
import java.sql.*;

import fbg.jdbc.server.Protocol;

// JDBC Driver for FBG Data Server
// -------------------------------

/**
 * <P>A Connection represents a session with a specific
 * database. Within the context of a Connection, SQL statements are
 * executed and results are returned.
 *
 * <P>A Connection's database is able to provide information
 * describing its tables, its supported SQL grammar, its stored
 * procedures, the capabilities of this connection, etc. This
 * information is obtained with the getMetaData method.
 *
 * <P><B>Note:</B> By default the Connection automatically commits
 * changes after executing each statement. If auto commit has been
 * disabled, an explicit commit must be done or database changes will
 * not be saved.
 *
 * @see DriverManager#getConnection
 * @see Statement
 * @see ResultSet
 * @see DatabaseMetaData
 */

public class ClientConnection implements Connection
{
  // Variables de instancia
  // ----------------------
  
  // Localizacin de la base de datos

  String host; 
  int    port;
  String db;  

  // Conexin

  String user;
  String password;

  Link   link = null;


  // Constructor
  // -----------

  public ClientConnection (String host, int port, String db, String user, String password)
         throws SQLException
  {
    String answer;

    this.host     = host;
    this.port     = port;
    this.db       = db;
    this.user     = user;
    this.password = password;

    try {

      link = new Link(host,port);

      while (!link.receive().equals(""))
            ;

      link.send(Protocol.LOGIN+"\n");
      link.send(db+"\n");
      link.send(user+"\n");
      link.send(password+"\n");
      link.send("\n");

      answer = link.receive();

      while (!link.receive().equals(""))
            ;

    } catch (IOException error) {

      link = null;
      throw new SQLException("JDBC CLIENT: "+error.toString());
    }

    if (!answer.equals(Protocol.MESSAGE))
       throw new SQLException("JDBC CLIENT: Connection refused by Data Server");
  }

  // toString

  public String toString ()
  {
    return host+":"+port+":"+db;
  }

  /**
   * SQL statements without parameters are normally
   * executed using Statement objects. If the same SQL statement
   * is executed many times, it is more efficient to use a
   * PreparedStatement
   *
   * @return a new Statement object
   * @exception SQLException if a database-access error occurs.
   */

  public Statement createStatement ()
                   throws SQLException
  {
    return new ClientStatement(this);
  }

  /**
   * A SQL statement with or without IN parameters can be
   * pre-compiled and stored in a PreparedStatement object. This
   * object can then be used to efficiently execute this statement
   * multiple times.
   *
   * <P><B>Note:</B> This method is optimized for handling
   * parametric SQL statements that benefit from precompilation. If
   * the driver supports precompilation, prepareStatement will send
   * the statement to the database for precompilation. Some drivers
   * may not support precompilation. In this case, the statement may
   * not be sent to the database until the PreparedStatement is
   * executed.  This has no direct affect on users; however, it does
   * affect which method throws certain SQLExceptions.
   *
   * @param sql a SQL statement that may contain one or more '?' IN
   * parameter placeholders
   * @return a new PreparedStatement object containing the
   * pre-compiled statement
   * @exception SQLException if a database-access error occurs.
   */

  public PreparedStatement prepareStatement (String sql)
                           throws SQLException
  {
    return new ClientPreparedStatement(this, sql);
  }

  /**
   * A SQL stored procedure call statement is handled by creating a
   * CallableStatement for it. The CallableStatement provides
   * methods for setting up its IN and OUT parameters, and
   * methods for executing it.
   *
   * <P><B>Note:</B> This method is optimized for handling stored
   * procedure call statements. Some drivers may send the call
   * statement to the database when the prepareCall is done; others
   * may wait until the CallableStatement is executed. This has no
   * direct affect on users; however, it does affect which method
   * throws certain SQLExceptions.
   *
   * @param sql a SQL statement that may contain one or more '?'
   * parameter placeholders. Typically this  statement is a JDBC
   * function call escape string.
   * @return a new CallableStatement object containing the
   * pre-compiled SQL statement
   * @exception SQLException if a database-access error occurs.
   */

  public CallableStatement prepareCall(String sql)
                           throws SQLException
  {
    // Not supported in DataServer

    return null;
  }

  /**
   * A driver may convert the JDBC sql grammar into its system's
   * native SQL grammar prior to sending it; nativeSQL returns the
   * native form of the statement that the driver would have sent.
   *
   * @param sql a SQL statement that may contain one or more '?'
   * parameter placeholders
   * @return the native form of this statement
   * @exception SQLException if a database-access error occurs.
   */

  public String nativeSQL (String sql)
                throws SQLException
  {
    return sql;
  }

  /**
   * If a connection is in auto-commit mode, then all its SQL
   * statements will be executed and committed as individual
   * transactions.  Otherwise, its SQL statements are grouped into
   * transactions that are terminated by either commit() or
   * rollback().  By default, new connections are in auto-commit
   * mode.
   *
   * The commit occurs when the statement completes or the next
   * execute occurs, whichever comes first. In the case of
   * statements returning a ResultSet, the statement completes when
   * the last row of the ResultSet has been retrieved or the
   * ResultSet has been closed. In advanced cases, a single
   * statement may return multiple results as well as output
   * parameter values. Here the commit occurs when all results and
   * output param values have been retrieved.
   *
   * @param autoCommit true enables auto-commit; false disables
   * auto-commit.
   * @exception SQLException if a database-access error occurs.
   */

  public void setAutoCommit (boolean autoCommit)
              throws SQLException
  {
    // Not supported
  }

  /**
   * Get the current auto-commit state.
   *
   * @return Current state of auto-commit mode.
   * @exception SQLException if a database-access error occurs.
   * @see #setAutoCommit
   */

  public boolean getAutoCommit()
                 throws SQLException
  {
    return true;
  }

  /**
   * Commit makes all changes made since the previous
   * commit/rollback permanent and releases any database locks
   * currently held by the Connection. This method should only be
   * used when auto commit has been disabled.
   *
   * @exception SQLException if a database-access error occurs.
   * @see #setAutoCommit
   */

  public void commit()
       throws SQLException
  {
    // not supported
  }

  /**
   * Rollback drops all changes made since the previous
   * commit/rollback and releases any database locks currently held
   * by the Connection. This method should only be used when auto
   * commit has been disabled.
   *
   * @exception SQLException if a database-access error occurs.
   * @see #setAutoCommit
   */

  public void rollback()
       throws SQLException
  {
    // not supported
  }

  /**
   * In some cases, it is desirable to immediately release a
   * Connection's database and JDBC resources instead of waiting for
   * them to be automatically released; the close method provides this
   * immediate release.
   *
   * <P><B>Note:</B> A Connection is automatically closed when it is
   * garbage collected. Certain fatal errors also result in a closed
   * Connection.
   *
   * @exception SQLException if a database-access error occurs.
   */

  public void close()
       throws SQLException
  {
    if (!isClosed()) {

       try {
         link.send( Protocol.CLOSE + "\n\n");
       } catch (IOException error) {
         throw new SQLException("JDBC CLIENT: "+error.toString());
       }

       link = null;
    }
  }

  /**
   * Tests to see if a Connection is closed.
   *
   * @return true if the connection is closed; false if it's still open
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isClosed()
          throws SQLException
  {
    return (link==null);
  }

  //======================================================================
  // Advanced features:

  /**
   * A Connection's database is able to provide information
   * describing its tables, its supported SQL grammar, its stored
   * procedures, the capabilities of this connection, etc. This
   * information is made available through a DatabaseMetaData
   * object.
   *
   * @return a DatabaseMetaData object for this Connection
   * @exception SQLException if a database-access error occurs.
   */

  public DatabaseMetaData getMetaData()
                   throws SQLException
  {
    return new ClientDatabaseMetaData(this);
  }

  /**
   * You can put a connection in read-only mode as a hint to enable
   * database optimizations.
   *
   * <P><B>Note:</B> setReadOnly cannot be called while in the
   * middle of a transaction.
   *
   * @param readOnly true enables read-only mode; false disables
   * read-only mode.
   * @exception SQLException if a database-access error occurs.
   */

  public void setReadOnly (boolean readOnly)
              throws SQLException
  {
    // not supported
  }

  /**
   * Tests to see if the connection is in read-only mode.
   *
   * @return true if connection is read-only
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isReadOnly()
          throws SQLException
  {
    return true;
  }

  /**
   * A sub-space of this Connection's database may be selected by setting a
   * catalog name. If the driver does not support catalogs it will
   * silently ignore this request.
   *
   * @exception SQLException if a database-access error occurs.
   */

  public void setCatalog(String catalog)
       throws SQLException
  {
    // not supported
  }

  /**
   * Return the Connection's current catalog name.
   *
   * @return the current catalog name or null
   * @exception SQLException if a database-access error occurs.
   */

  public String getCatalog()
         throws SQLException
  {
    return null;
  }


  // TRANSACTIONS
  // ------------

  /**
   * Transactions are not supported.
   */

  public int TRANSACTION_NONE	     = 0;

  /**
   * Dirty reads, non-repeatable reads and phantom reads can occur.
   */

  public int TRANSACTION_READ_UNCOMMITTED = 1;

  /**
   * Dirty reads are prevented; non-repeatable reads and phantom
   * reads can occur.
   */

  public int TRANSACTION_READ_COMMITTED   = 2;

  /**
   * Dirty reads and non-repeatable reads are prevented; phantom
   * reads can occur.
   */

  public int TRANSACTION_REPEATABLE_READ  = 4;

  /**
   * Dirty reads, non-repeatable reads and phantom reads are prevented.
   */

  public int TRANSACTION_SERIALIZABLE     = 8;

  /**
   * You can call this method to try to change the transaction
   * isolation level using one of the TRANSACTION_* values.
   *
   * <P><B>Note:</B> setTransactionIsolation cannot be called while
   * in the middle of a transaction.
   *
   * @param level one of the TRANSACTION_* isolation values with the
   * exception of TRANSACTION_NONE; some databases may not support
   * other values
   * @exception SQLException if a database-access error occurs.
   * @see DatabaseMetaData#supportsTransactionIsolationLevel
   */

  public void setTransactionIsolation(int level)
       throws SQLException
  {
    // not supported
  }

  /**
   * Get this Connection's current transaction isolation mode.
   *
   * @return the current TRANSACTION_* mode value
   * @exception SQLException if a database-access error occurs.
   */

  public int getTransactionIsolation()
      throws SQLException
  {
    return TRANSACTION_NONE;
  }

  /**
   * The first warning reported by calls on this Connection is
   * returned.
   *
   * <P><B>Note:</B> Subsequent warnings will be chained to this
   * SQLWarning.
   *
   * @return the first SQLWarning or null
   * @exception SQLException if a database-access error occurs.
   */

  public SQLWarning getWarnings()
             throws SQLException
  {
    return null;
  }

  /**
   * After this call, getWarnings returns null until a new warning is
   * reported for this Connection.
   *
   * @exception SQLException if a database-access error occurs.
   */

  public void clearWarnings()
       throws SQLException
  {
  }

    //--------------------------JDBC 2.0-----------------------------

    /**
     * JDBC 2.0
     *
	 * Creates a <code>Statement</code> object that will generate
	 * <code>ResultSet</code> objects with the given type and concurrency.
     * This method is the same as the <code>createStatement</code> method
	 * above, but it allows the default result set
     * type and result set concurrency type to be overridden.
     *
     * @param resultSetType a result set type; see ResultSet.TYPE_XXX
     * @param resultSetConcurrency a concurrency type; see ResultSet.CONCUR_XXX
     * @return a new Statement object 
     * @exception SQLException if a database access error occurs
     */
    
     public Statement createStatement(int resultSetType, int resultSetConcurrency) 
                      throws SQLException
     {
       return null;
     }

    /**
     * JDBC 2.0
     *
	 * Creates a <code>PreparedStatement</code> object that will generate
	 * <code>ResultSet</code> objects with the given type and concurrency.
     * This method is the same as the <code>prepareStatement</code> method
	 * above, but it allows the default result set
     * type and result set concurrency type to be overridden.
     *
     * @param resultSetType a result set type; see ResultSet.TYPE_XXX
     * @param resultSetConcurrency a concurrency type; see ResultSet.CONCUR_XXX
     * @return a new PreparedStatement object containing the
     * pre-compiled SQL statement 
     * @exception SQLException if a database access error occurs
     */

     public PreparedStatement prepareStatement ( String sql, 
                                                 int resultSetType, 
					                             int resultSetConcurrency)
                              throws SQLException
     {
       return null;
     }

    /**
     * JDBC 2.0
     *
	 * Creates a <code>CallableStatement</code> object that will generate
	 * <code>ResultSet</code> objects with the given type and concurrency.
     * This method is the same as the <code>prepareCall</code> method
	 * above, but it allows the default result set
     * type and result set concurrency type to be overridden.
     *
     * @param resultSetType a result set type; see ResultSet.TYPE_XXX
     * @param resultSetConcurrency a concurrency type; see ResultSet.CONCUR_XXX
     * @return a new CallableStatement object containing the
     * pre-compiled SQL statement 
     * @exception SQLException if a database access error occurs
     */

    public CallableStatement prepareCall ( String sql, 
                                           int resultSetType, 
				                           int resultSetConcurrency)
                             throws SQLException
    {
      return null;
    }

    /**
     * JDBC 2.0
     *
     * Gets the type map object associated with this connection.
     * Unless the application has added an entry to the type map,
	 * the map returned will be empty.
	 *
	 * @return the <code>java.util.Map</code> object associated 
	 *         with this <code>Connection</code> object
     */
    
    public java.util.Map getTypeMap() throws SQLException
    {
      return null;
    }

    /**
     * JDBC 2.0
     *
     * Installs the given type map as the type map for
     * this connection.  The type map will be used for the
	 * custom mapping of SQL structured types and distinct types.
	 *
	 * @param the <code>java.util.Map</code> object to install
	 *        as the replacement for this <code>Connection</code>
	 *        object's default type map
     */

    public void setTypeMap(java.util.Map map) throws SQLException
    {
    }

}
