package fbg.jdbc.client;

// Title:       JDBC 2.0 Driver for FBG Data Server
// Version:     1.0
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.io.*;
import java.sql.*;


/**
 * A ResultSetMetaData object can be used to find out about the types 
 * and properties of the columns in a ResultSet.
 */

public class ClientResultSetMetaData
       implements ResultSetMetaData
{
  ClientConnection connect;
  ClientResultSet  set;

  int    size;
  String attr[];
  String type[];
  
  // Constructor

  public ClientResultSetMetaData (ClientConnection connect, ClientResultSet set)
         throws SQLException
  {
    int i;

    this.connect = connect;
    this.set     = set;

    try {

      size = Integer.parseInt(connect.link.receive());

      attr = new String[size];
      type = new String[size];

      for (i=0; i<size; i++) {
          attr[i] = connect.link.receive().toUpperCase();
          type[i] = connect.link.receive().toUpperCase();
      }

    } catch (IOException error) {
      throw new SQLException("JDBC Client: "+error.toString());
    }
  }

  /**
   * What's the number of columns in the ResultSet?
   *
   * @return the number
   * @exception SQLException if a database-access error occurs.
   */

  public int getColumnCount() throws SQLException
  {
    return size;
  }

  /**
   * Is the column automatically numbered, thus read-only?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isAutoIncrement(int column) throws SQLException
  {
    return true;
  }

  /**
   * Does a column's case matter?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isCaseSensitive(int column) throws SQLException
  {
    return true;
  }

  /**
   * Can the column be used in a where clause?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isSearchable(int column) throws SQLException
  {
    return true;
  }

  /**
   * Is the column a cash value?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isCurrency(int column) throws SQLException
  {
    return false;
  }

  /**
   * Can you put a NULL in this column?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return columnNoNulls, columnNullable or columnNullableUnknown
   * @exception SQLException if a database-access error occurs.
   */

  public int isNullable(int column) throws SQLException
  {
    return columnNullable;
  }

    /**
     * Does not allow NULL values.
     */
    public int columnNoNulls = 0;

    /**
     * Allows NULL values.
     */
    public int columnNullable = 1;

    /**
     * Nullability unknown.
     */
    public int columnNullableUnknown = 2;

  /**
   * Is the column a signed number?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isSigned(int column) throws SQLException
  {
    if (type[column-1].startsWith("NUMBER"))
       return true;
    else
       return false;
  }

  /**
   * What's the column's normal max width in chars?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return max width
   * @exception SQLException if a database-access error occurs.
   */

  public int getColumnDisplaySize(int column) throws SQLException
  {
    return 10;
  }

  /**
   * What's the suggested column title for use in printouts and
   * displays?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public String getColumnLabel(int column) throws SQLException
  {
    return attr[column-1];
  }

  /**
   * What's a column's name?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return column name
   * @exception SQLException if a database-access error occurs.
   */

  public String getColumnName(int column) throws SQLException
  {
    return attr[column-1];
  }

  /**
   * What's a column's table's schema?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return schema name or "" if not applicable
   * @exception SQLException if a database-access error occurs.
   */

  public String getSchemaName(int column) throws SQLException
  {
    return null;
  }

  /**
   * What's a column's number of decimal digits?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return precision
   * @exception SQLException if a database-access error occurs.
   */

  public int getPrecision(int column) throws SQLException
  {
    return 0;
  }

  /**
   * What's a column's number of digits to right of the decimal point?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return scale
   * @exception SQLException if a database-access error occurs.
   */

  public int getScale(int column) throws SQLException
  {
    return 0;
  }

  /**
   * What's a column's table name?
   *
   * @return table name or "" if not applicable
   * @exception SQLException if a database-access error occurs.
   */

  public String getTableName(int column) throws SQLException
  {
    return "";
  }

  /**
   * What's a column's table's catalog name?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return column name or "" if not applicable.
   * @exception SQLException if a database-access error occurs.
   */

  public String getCatalogName(int column) throws SQLException
  {
    return "";
  }

  /**
   * What's a column's SQL type?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return SQL type
   * @exception SQLException if a database-access error occurs.
   * @see Types
   */

  public int getColumnType(int column) throws SQLException
  {
    int tipo;

    if (type[column-1].startsWith("NUMBER")) {
       tipo = java.sql.Types.NUMERIC;
    } else {
       tipo = java.sql.Types.CHAR;
    }

    return tipo;
  }

  /**
   * What's a column's data source specific type name?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return type name
   * @exception SQLException if a database-access error occurs.
   */

  public String getColumnTypeName(int column) throws SQLException
  {
    return type[column-1];
  }

  /**
   * Is a column definitely not writable?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isReadOnly(int column) throws SQLException
  {
    return true;
  }

  /**
   * Is it possible for a write on the column to succeed?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */

  public boolean isWritable(int column) throws SQLException
  {
    return false;
  }

  /**
   * Will a write on the column definitely succeed?
   *
   * @param column the first column is 1, the second is 2, ...
   * @return true if so
   * @exception SQLException if a database-access error occurs.
   */
   
   public boolean isDefinitelyWritable(int column) throws SQLException
   {
     return false;
   }

    //--------------------------JDBC 2.0-----------------------------------

    /**
     * JDBC 2.0
     *
     * <p>Returns the fully-qualified name of the Java class whose instances 
     * are manufactured if the method <code>ResultSet.getObject</code>
	 * is called to retrieve a value 
     * from the column.  <code>ResultSet.getObject</code> may return a subclass of the
     * class returned by this method.
	 *
	 * @return the fully-qualified name of the class in the Java programming
	 *         language that would be used by the method 
	 * <code>ResultSet.getObject</code> to retrieve the value in the specified
	 * column. This is the class name used for custom mapping.
     * @exception SQLException if a database access error occurs
     */

    public String getColumnClassName(int column) throws SQLException
    {
      return null;
    }
}
