package fbg.util;

// Title:       Confioguration file
// Version:     1.0
// Copyright:   February 1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.io.File;
import java.io.RandomAccessFile;
import java.io.IOException;

import java.util.Vector;

/**
 * <p>
 * Represents a configuration file.
 *
 * <p>
 * Format similar to MS Windows INI files:
 * <UL>
 * <LI> Sections are headed by lines beginning with <code>[</code> or <code>#</code>.
 * <LI> Comments are tagged with <code>;</code> at the beginning of each line.
 * </UL>
 *
 * @version 1.0
 * @author  Fernando Berzal Galiano
 */

public class ConfigFile
{
  File   filename;
  Vector sections;

  // Constructor

  /**
   * Loads a configuration file.
   *
   * @param     fileID  file name
   */

  public ConfigFile (String fileID)
  {
    String           sectionID = null;
    RandomAccessFile file      = null;

    sections = new Vector();

    try {

        filename = new File(fileID);

        file     = new RandomAccessFile (filename, "r" );

        do {

           sectionID = readASCII(file);

        } while (  sectionID!=null
                && (sectionID.length()==0 || sectionID.charAt(0)==';') );

        while (sectionID!=null) {

              sectionID = readSection (file, sectionID);
        }

        file.close();

    } catch (IOException e) {

        System.out.println("ERROR: " + e.getMessage());
    }
  }

  // Section load
  // ------------
  // - Returns next section ID

  private String readSection (RandomAccessFile file, String sectionID)
                 throws IOException
  {
    Vector  section = new Vector();
    String  ascii;
    boolean eos = false;                // End Of Section flag

    section.addElement(sectionID.toUpperCase());

    do {

       ascii = readASCII(file);

       if (ascii!=null) {

          if (ascii.length()>0) {

             if (  ascii.charAt(0)=='['
                || ascii.charAt(0)=='#' ) {

                // Beginning of the next section

                eos = true;

             } else if ( ascii.charAt(0)==';' ) {

                // Comment

             } else {

                section.addElement(ascii);
             }
          }

       } else {

          eos = true;
       }

    } while (!eos);


    sections.addElement(section);

    return ascii;
  }

  // Get an ASCII String from RandomAccessFile

  private String readASCII (RandomAccessFile file)
                 throws IOException
  {
    String ascii = file.readLine();

    if (ascii!=null)
       ascii = ascii.trim();

    return ascii;
  }

  // Access to a given section

  /**
   * Gets a section of the configuration file.
   *
   * @param     ID      section identifier
   * @return            String array containing section text
   */

  public String[] Section (String ID)
  {
    Vector  section;
    String  array[] = null;
    String  id;
    int     size  = sections.size();
    int     pos   = 0;
    int     i;
    boolean found = false;


    while (pos<size && !found) {

          section = (Vector) sections.elementAt(pos);
          id = (String) section.elementAt(0);

          if (id.equals(ID)) {

             array = new String[section.size()-1];

             for (i=0; i<array.length; i++)
                 array[i] = (String) section.elementAt(i+1);
          }

          pos++;
    }

    return array;
  }

  /**
   * Gets a String from a section of the configuration file.
   *
   * @param     ID      section identifier
   * @return            First line of section contents
   */

  public String getString (String ID)
  {
    String array[] = Section (ID);

    if (array!=null && array.length>0)
       return array[0];
    else
       return null;
  }

  /**
   * Gets an integer from a section of the configuration file.
   *
   * @param     ID      section identifier
   * @return            Integer found in the first line of section text
   *                    (on error returns <code>0</code>).
   */

  public int getInt ( String ID )
  {
    String array[] = Section (ID);

    if (array!=null && array.length>0)
       return (new Integer(array[0])).intValue();
    else
       return 0;
  }


  // File information
  // ----------------

  /**
   * Gets the directory of the configuration file.
   */

  public String path ()
  {
    return filename.getParent();
  }


  // Standard output
  // ---------------

  /**
   * Standard output.
   */

  public void print ()
  {
    int  size  = sections.size();
    int  pos   = 0;

    for (pos=0; pos<size; pos++)
        printSection( (Vector) sections.elementAt(pos) );

  }

  private void printSection (Vector section)
  {
    int  size  = section.size();
    int  pos   = 0;

    // System.out.println("; Section ID");
    System.out.println( (String) section.elementAt(0) );
    // System.out.println("; Section contents");

    for (pos=1; pos<size; pos++)
        System.out.println( (String) section.elementAt(pos) );

  }


  // TEST

  /**
   * Test program.
   * Reads a configuration file and outputs its sections.
   */

  public static void main (String args[])
  {
    ConfigFile cf = new ConfigFile(args[0]);

    cf.print();
  }

}