package fbg.util.db;

// Title:       TMiner
// Version:     1.1
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.util.Vector;

import fbg.util.math.Decimal;

// CLUSTERING
// ----------

// Versin 1.0 - Septiembre 1998 - 256 clusters mximo
// Versin 1.1 - Mayo       1999 - fbg.util.math.Decimal

public class Clustering
{
  // Parmetros

  public static final int MAX_ITERACIONES = 100;

  // Clusters

  Vector min = null;
  Vector max = null;


  // Algoritmos disponibles
  // ----------------------

  public static final byte EQUI_DEPTH = 1;
  public static final byte KMEANS     = 2;
  public static final byte MST        = 3;
  public static final byte MAQA       = 4;

  // Constructor
  // -----------

  public Clustering (byte algoritmo, int clusters, Valores valores)
  {
    min = new Vector();
    max = new Vector();

    if (valores.Total()>clusters) {

       switch (algoritmo) {

         case EQUI_DEPTH:
              EquiDepthClustering (clusters, valores, min, max);
              break;

         case KMEANS:
              KMeans (clusters, valores, min, max);
              break;

         case MST:
              MSTClustering (clusters, valores, min, max);
              break;

         case MAQA:
              MAQA (clusters, valores, min, max);
              break;

         default:
              min.addElement( (Decimal) valores.Valor(0).valor );
              max.addElement( (Decimal) valores.Valor(valores.Total()-1).valor );
       }

    } else {

       int i;

       for (i=0; i<valores.Total(); i++) {
           min.addElement( (Decimal) valores.Valor(i).valor );
           max.addElement( (Decimal) valores.Valor(i).valor );
       }
    }
  }

  // Nmero de clusters
  // ------------------

  public int clusters ()
  {
    return min.size();  // == max.size();
  }

  // Descripcin de los clusters
  // ---------------------------

  public Decimal min (int c)
  {
    if (c>=0 && c<clusters()) {

       return (Decimal) min.elementAt(c);

    } else {

       return null;
    }
  }

  public Decimal max (int c)
  {
    if (c>=0 && c<clusters()) {

       return (Decimal) max.elementAt(c);

    } else {

       return null;
    }
  }

  // Rutinas auxiliares
  // ------------------

  // Valor numrico del valor i

  private double v (Valores valores, int i)
  {
    return ((Decimal)valores.Valor(i).valor).doubleValue();
  }

  // Frecuencia de aparicin del valor i

  private int h (Valores valores, int i)
  {
    return valores.Valor(i).support;
  }

  // Distancia de un valor a otro

  private double d (double a, double b)
  {
    double dif = a-b;

    return dif*dif;
  }

  // Equi-depth Clustering
  // ---------------------

  private void EquiDepthClustering ( int     clusters,
                                     Valores valores,
                                     Vector  min,
                                     Vector  max )
  {
    int   i;
    int   total;
    int   minimo;
    float limit;
    float suma;

    // Nmero de valores por cluster

    suma = 0;

    for (i=0; i<valores.Total(); i++)
        suma += h(valores,i);

    suma /= clusters;

    // Recorrido de los valores

    minimo = 0;
    total  = 0;
    limit  = suma;

    for (i=0; i<valores.Total(); i++) {

        total += h(valores,i);

        if (total>=limit) {
           min.addElement((Decimal) valores.Valor(minimo).valor );
           max.addElement((Decimal) valores.Valor(i).valor );

           minimo = i+1;

           while (total>=limit)
                 limit+=suma;
        }
    }
  }

  // KM (K-MEANS)
  // ------------
  // Minimization of the sum of squared distance
  //
  // Variante: ISODATA (Iterative Self-Organizing Data Analysis Techniques)

  private void KMeans ( int     clusters,
                        Valores valores,
                        Vector  min,
                        Vector  max )
  {
    int    i;
    int    total     = valores.Total();
    byte   cluster[] = new byte[total];

    int    size[]    = new int[clusters];
    double center[]  = new double[clusters];
    double old[]     = new double[clusters];

    int     iteracion;
    byte    id;
    int     freq;
    double  valor;
    double  minimo;
    double  dist;
    boolean cambios;

    // Configuracin inicial de los clusters

    for (i=0; i<clusters; i++) {
        center[i] = 0;
        size[i] = 0;
    }

    dist = (double)clusters/(double)total;

    for (i=0; i<total; i++) {
        valor = v(valores,i);
        freq  = h(valores,i);
        id    = (byte)(i*dist); // de 0 a (clusters-1)

        cluster[i] =  id;
        size[id]   += freq;
        center[id] += freq*valor;
    }

    for (i=0; i<clusters; i++)
        if (size[i]>0)
           center[i] /= size[i];

    // Algoritmo iterativo

    iteracion = 0;

    do {

       for (i=0; i<clusters; i++) {
           old[i]    = center[i];
           size[i]   = 0;
           center[i] = 0;
       }

       for (i=0; i<total; i++) {

           valor  = v(valores,i);
           freq   = h(valores,i);

           cluster[i] = 0;
           minimo     = d(valor,old[0]);

           for (id=1; id<clusters; id++) {
               dist = d(valor,old[id]);

               if (dist<minimo) {
                  cluster[i] = id;
                  minimo     = dist;
               }
           }

           size[cluster[i]]   += freq;
           center[cluster[i]] += freq*valor;
       }

       for (i=0; i<clusters; i++)
           if (size[i]>0)
              center[i] /= size[i];

       cambios = false;

       for (i=0; i<clusters && !cambios; i++)
           if (center[i]!=old[i])
              cambios = true;

       iteracion++;

    } while (cambios && iteracion<MAX_ITERACIONES);

    // Creacin de los clusters

    min.addElement((Decimal) valores.Valor(0).valor);

    for (i=1; i<total; i++) {
        if (cluster[i]!=cluster[i-1]) {
           max.addElement((Decimal) valores.Valor(i-1).valor);
           min.addElement((Decimal) valores.Valor(i).valor);
        }
    }

    max.addElement( (Decimal) valores.Valor(valores.Total()-1).valor );
  }

  // MST CLUSTERING
  // --------------
  // Minimal Spanning Tree
  //
  // Variantes: Shared Near Neighbor Maximal Spanning Tree

  private void MSTClustering ( int     clusters,
                               Valores valores,
                               Vector  min,
                               Vector  max )
  {
    int    i,j;
    int    total   = valores.Total();
    int    aristas = clusters-1;
    int    limit[] = new int[aristas];     // x == Arista entre (x) y (x+1)
    double arist[] = new double[aristas];  // d(x,x+1);
    int    minpos;
    double minval;
    double dist;

    // (clusters-1) aristas de mayor longitud del rbol generador minimal

    for (i=0; i<aristas; i++) {
        limit[i] = i;
        arist[i] = d(v(valores,i),v(valores,i+1));
    }

    minpos = 0;
    minval = arist[0];

    for (j=1; j<aristas; j++) {
        if (arist[j]<minval) {
           minpos = j;
           minval = arist[j];
        }
    }

    for (i=aristas; i<total-1; i++) {

        dist = d(v(valores,i),v(valores,i+1));

        if (dist>minval) {

           for (j=minpos; j<aristas-1; j++) {
               limit[j] = limit[j+1];
               arist[j] = arist[j+1];
           }

           limit[aristas-1] = i;
           arist[aristas-1] = dist;

           minpos = 0;
           minval = arist[0];

           for (j=1; j<aristas; j++) {
               if (arist[j]<minval) {
                  minpos = j;
                  minval = arist[j];
               }
           }
        }
    }

    // Creacin de los clusters

    min.addElement((Decimal) valores.Valor(0).valor);

    for (i=0; i<aristas; i++) {
        max.addElement((Decimal) valores.Valor(limit[i]).valor);
        min.addElement((Decimal) valores.Valor(limit[i]+1).valor);
    }

    max.addElement( (Decimal) valores.Valor(total-1).valor );
  }

  // MAQA CLUSTERING
  // ---------------
  // Zhaohui Zhang, Yuchang Lu & Bo Zhang:
  // "An effective partitioning-combining algorithm
  //  for discovering quantitative association rules"
  //
  // En "KDD. Techniques and Applications"
  //    H. Lu et al. eds.
  //    World Scientific, 1997.
  //
  // Variacin: Se fija la cota superior del nmero de clusters

  private void MAQA ( int     clusters,
                      Valores valores,
                      Vector  min,
                      Vector  max )
  {
    final byte NONE = 0;
    final byte MAX  = 1;
    final byte MIN  = 2;

    int    i;
    int    total  = valores.Total();

    int    nmin;
    int    nval;
    int    hval;
    int    hprev;
    double suma;
    double dens;
    double dmin;
    Vector deni = new Vector(); // SUMi / (MAXi - MINi)

    // Inicializacin

    nmin = 0;

    while (nmin<total) {

          nval  = nmin;
          hval  = h(valores,nval);
          suma  = 0;

          do {
             suma += hval;
             hprev = hval;
             nval ++;

             if (nval<total)
                hval = h(valores,nval);

          } while (nval<total && hval>=hprev);

          if (nval<total)
             do {
                 suma += hval;
                 hprev = hval;
                 nval ++;

                 if (nval<total)
                    hval = h(valores,nval);

             } while (nval<total && hval<=hprev);

          // Intervalo [nmin,nval-1]

          min.addElement((Decimal) valores.Valor(nmin).valor);
          max.addElement((Decimal) valores.Valor(nval-1).valor);

          if (nval-1==nmin)
             dens = suma;
          else
             dens = suma/(v(valores,nval-1)-v(valores,nmin));

          deni.addElement(new Double(dens));

          nmin = nval;
    }

    // Eliminacin de los intervalos menos "densos"

    while (deni.size()>clusters) {

          nmin = 0;
          dmin = ((Double)deni.elementAt(0)).doubleValue();

          for (i=1; i<deni.size(); i++) {
              dens = ((Double)deni.elementAt(i)).doubleValue();

              if (dens<dmin) {
                 nmin = i;
                 dmin = dens;
              }
          }

          max.removeElementAt(nmin);
          min.removeElementAt(nmin);
          deni.removeElementAt(nmin);
    }
  }

}