package fbg.util.db;

// Title:       TMiner
// Version:     1.1
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import fbg.util.math.Decimal;

// THRESHOLDING: Tcnicas de umbralizacin automtica
// --------------------------------------------------

// Versin 1.0: Agosto 1998 - 10 algoritmos diferentes
// Versin 1.1: Mayo   1999 - fbg.util.math.Decimal

public class Thresholding
{
  int umbral;

  // Algoritmos disponibles
  // ----------------------

  public static final byte MEAN                = 1;
  public static final byte TWO_PEAKS           = 2;
  public static final byte ITERATIVE_SELECTION = 3;
  public static final byte OTSU                = 4;
  public static final byte PUN                 = 5;
  public static final byte KAPUR               = 6;
  public static final byte JOHANNSEN           = 7;
  public static final byte FUZZY_ENTROPY       = 8;
  public static final byte FUZZY_YAGER         = 9;
  public static final byte MINIMUM_ERROR       = 10;

  // Constructor
  // -----------

  public Thresholding (byte algoritmo, Valores valores)
  {
    umbral = -1;

    switch (algoritmo) {

      case MEAN:
           umbral = MeanThresholding (valores);
           break;

      case TWO_PEAKS:
           umbral = TwoPeaksThresholding (valores);
           break;

      case ITERATIVE_SELECTION:
           umbral = IterativeSelectionThresholding (valores);
           break;

      case OTSU:
           umbral = OtsuThresholding (valores);
           break;

      case PUN:
           umbral = PunThresholding (valores);
           break;

      case KAPUR:
           umbral = KapurThresholding (valores);
           break;

      case JOHANNSEN:
           umbral = JohannsenThresholding (valores);
           break;

      case FUZZY_ENTROPY:
      case FUZZY_YAGER:
           umbral = FuzzyThresholding (valores, algoritmo);
           break;

      case MINIMUM_ERROR:
           umbral = MinimumErrorThresholding (valores);
           break;
    }
  }

  // Rutinas auxiliares
  // ------------------

  private double v (Valores valores, int i)
  {
    return ((Decimal)valores.Valor(i).valor).doubleValue();
  }

  private int h (Valores valores, int i)
  {
    return valores.Valor(i).support;
  }

  private int n (Valores valores, double t)
  {
    int   i;
    int   total = valores.Total();
    Valor valor;

    for (i=1; i<total; i++) {
        valor = valores.Valor(i);

        if (((Decimal)valor.valor).doubleValue()>t)
           return i-1;
    }

    return -1;
  }

  // Media
  // -----

  private int MeanThresholding (Valores valores)
  {
    int    i;
    int    total = valores.Total();
    double media = 0;
    int    num   = 0;
    Valor  valor;

    for (i=0; i<total; i++) {
        valor = valores.Valor(i);
        num   += valor.support;
        media += valor.support*((Decimal)valor.valor).doubleValue();
    }

    media /= num;

    for (i=1; i<total; i++) {
        valor = valores.Valor(i);

        if (((Decimal)valor.valor).doubleValue()>media)
           return i-1;
    }

    return -1;
  }

  // Picos del histograma
  // --------------------

  private int TwoPeaksThresholding (Valores valores)
  {
    int    i,j,k,t;
    int    total = valores.Total();
    double vj,kj,ij;

    // Primer pico del histograma

    j = 0;

    for (i=0; i<total; i++)
        if (valores.Valor(i).support > valores.Valor(j).support)
           j = i;

    vj = ((Decimal)valores.Valor(j).valor).doubleValue();

    // Segundo pico del histograma

    k = 0;

    for (i=1; i<total-1; i++)
        if (  valores.Valor(i-1).support<=valores.Valor(i).support
           && valores.Valor(i+1).support<=valores.Valor(i).support ) {

           kj = ((Decimal)valores.Valor(k).valor).doubleValue() - vj;
           ij = ((Decimal)valores.Valor(i).valor).doubleValue() - vj;

           if (kj*kj*valores.Valor(k).support < ij*ij*valores.Valor(i).support )
              k = i;
        }

    // Umbral

    t = j;

    if (j<k) {

       for (i=j; i<k; i++)
           if ( valores.Valor(i).support < valores.Valor(t).support )
              t = i;

    } else {

       for (i=k; i<j; i++)
           if ( valores.Valor(i).support < valores.Valor(t).support )
              t = i;

    }

    return t;
  }

  // Seleccin iterativa
  // -------------------

  private int IterativeSelectionThresholding (Valores valores)
  {
    int    i;
    int    total = valores.Total();
    int    told;
    int    tt = MeanThresholding(valores);
    double a,b,c,d,vt;

    do {
        told = tt;

        a = 0;
        b = 0;

        for (i=0; v(valores,i)<=v(valores,told); i++) {
            a += v(valores,i)*h(valores,i);
            b += h(valores,i);
        }

        b += b;

        c = 0;
        d = 0;

        for (i=told+1; i<total; i++) {
            c += v(valores,i)*h(valores,i);
            d += h(valores,i);
        }

        d += d;

        if (b==0) b = 1;
        if (d==0) d = 1;

        vt = a/b + c/d;
        tt = n(valores,vt);

    } while (tt != told);

    return tt;
  }

  // Otsu Thresholding: Otsu's method of histograms
  // ----------------------------------------------

  private double w (double p[], int k)
  {
    int    i;
    double x=0.0;

    for (i=0; i<k; i++)
        x += p[i];

    return x;
  }

  private double u (Valores valores, double p[], int k)
  {
    int    i;
    double x=0.0;

    for (i=0; i<k; i++)
        x += v(valores,i)*p[i];

    return x;
  }

  private double nu (Valores v, double p[], int k, double ut, double vt)
  {
    double x, y;

    y = w(p,k);
    x = ut*y - u(v,p,k);
    x = x*x;
    y = y*(1.0-y);

    if (y>0)
       x = x/y;
    else
       x = 0.0;

    return x/vt;
  }

  private int OtsuThresholding (Valores valores)
  {
    int    total = valores.Total();
    int    suma;
    int    i,j,k,t;
    double y,z,ut,vt,dif;
    double p[] = new double[total];

    // Histograma normalizado

    suma = 0;

    for (i=0; i<total; i++) {
        p[i] = h(valores,i);
        suma += p[i];
    }

    for (i=0; i<total; i++)
        p[i] /= (double)suma;

    // Media global

    ut = u(valores,p,total);

    // Varianza global

    vt = 0.0;

    for (i=0; i<total; i++) {
        dif = v(valores,i)-ut;
        vt += dif*dif*p[i];
    }

    // Seleccin del umbral

    z = -1.0;
    t = 0;

    for (i=0; i<total; i++) {

        y = nu (valores, p, i, ut, vt);

        if (y>=z) {
           z = y;
           t = i;
        }
    }

    return t;
  }

  // Pun thresholding method using entropy
  // -------------------------------------

  private double entropy (double h[], int a)
  {
    if (h[a] > 0.0)
       return -h[a] * Math.log(h[a]);
    else
       return 0.0;
  }

  private double flog (double x)
  {
    if (x <= 0.0)
       return 0.0;
    else
       return Math.log(x);
  }

  private double maxtot (double h[], int i)
  {
    double x;
    int    j;

    x = h[0];

    for (j=1; j<=i; j++)
        if (x < h[j])
           x = h[j];

    return x;
  }

  private double maxfromt (double h[], int i)
  {
    double x;
    int    j;

    x = h[i+1];

    for (j=i+2; j<h.length; j++)
        if (x < h[j])
           x = h[j];

    return x;
  }

  private int PunThresholding (Valores valores)
  {
    int    i, j, t, suma;
    int    total  = valores.Total();
    double hist[] = new double[total];
    double Pt[]   = new double[total];
    double Ht[]   = new double[total];
    double F[]    = new double[total];
    double HT,x,y,z,to,from;

    // Histograma normalizado

    suma = 0;

    for (i=0; i<total; i++) {
        hist[i] = h(valores,i);
        suma += hist[i];
    }

    for (i=0; i<total; i++)
        hist[i] /= (double)suma;

    // Factores

    HT = Ht[0] = entropy (hist, 0);
    Pt[0] = hist[0];

    for (i=1; i<total; i++) {
        Pt[i] = Pt[i-1] + hist[i];
        x = entropy(hist, i);
        Ht[i] = Ht[i-1] + x;
        HT += x;
    }

    // Funcin que se debe maximizar

    t = 0;

    for (i=0; i<total-1; i++) {

        to   = maxtot(hist,i);
        from = maxfromt(hist, i);

        if (to > 0.0 && from > 0.0) {
           x = (Ht[i]/HT)* flog(Pt[i])/flog(to);
           y = 1.0 - (Ht[i]/HT);
           z = flog(1 - Pt[i])/flog(from);
        } else
           x = y = z = 0.0;

        F[i] = x + y*z;

        if (i>0 && F[i] > F[t])
           t = i;
    }

    return t;
  }

  // Kapur thresholding method using entropy
  // ---------------------------------------

  private double entropia (double h[], int a, double p)
  {
    if (h[a] > 0.0 && p>0.0)
       return -(h[a]/p * Math.log(h[a]/p));
    else
       return 0.0;
  }

  private int KapurThresholding (Valores valores)
  {
    int    i, j, t, suma;
    double Hb, Hw;

    int    total  = valores.Total();
    double hist[] = new double[total];
    double Pt[]   = new double[total];
    double F[]    = new double[total];

    // Histograma normalizado

    suma = 0;

    for (i=0; i<total; i++) {
        hist[i] = h(valores,i);
        suma += hist[i];
    }

    for (i=0; i<total; i++)
        hist[i] /= (double)suma;

    // Factores

    Pt[0] = hist[0];

    for (i=1; i<total; i++)
        Pt[i] = Pt[i-1] + hist[i];

    // Funcin que se debe maximizar

    t = 0;

    for (i=0; i<total-1; i++) {

        Hb = Hw = 0.0;

        for (j=0; j<total; j++)
            if (j<=i)
                Hb += entropia (hist, j, Pt[i]);
            else
                Hw += entropia (hist, j, 1.0-Pt[i]);

        F[i] = Hb+Hw;

        if (i>0 && F[i] > F[t])
           t = i;
    }

    return t;
  }

  // Johannsen thresholding method using entropy
  // -------------------------------------------

  private double entrop (double h)
  {
    if (h > 0.0)
       return -h * Math.log(h);
    else
       return 0.0;
  }

  private double doublelog (double x)
  {
    if (x > 0.0)
       return Math.log(x);
    else
       return 0.0;
  }

  private int JohannsenThresholding (Valores valores)
  {
    int    i, j, t, suma;
    double Sb, Sw;
    int    total  = valores.Total();
    double hist[] = new double[total];
    double Pt[]   = new double[total];
    double Pq[]   = new double[total];
    double F[]    = new double[total];

    // Histograma normalizado

    suma = 0;

    for (i=0; i<total; i++) {
        hist[i] = h(valores,i);
        suma += hist[i];
    }

    for (i=0; i<total; i++)
        hist[i] /= (double)suma;

    // Factores

    Pt[0] = hist[0];
    Pq[0] = 1.0 - Pt[0];

    for (i=1; i<total; i++) {
        Pt[i] = Pt[i-1] + hist[i];
        Pq[i] = 1.0 - Pt[i-1];
    }

    // Funcin que se debe minimizar

    t = -1;

    for (i=1; i<total-1; i++) {
        Sb = doublelog(Pt[i])+(1.0/Pt[i])*(entrop(hist[i])+entrop(Pt[i-1]));
        Sw = doublelog(Pq[i])+(1.0/Pq[i])*(entrop(hist[i])+entrop(Pq[i+1]));
        F[i] = Sb+Sw;

        if ((t<0) || (F[i] < F[t]))
           t = i;
    }

    return t;
  }

  // Fuzzy Thresholding
  // ------------------

  private double Ux (Valores valores, int g, int u0, int u1, int t)
  {
    double ux, x;
    double dif = v(valores,valores.Total()-1) - v(valores,0);

    if (g <= t) {
       x = 1.0 + Math.abs(v(valores,g) - v(valores,u0))/dif;
       ux = 1.0/x;
    } else {
       x = 1.0 + Math.abs(v(valores,g) - v(valores,u1))/dif;
       ux = 1.0/x;
    }

    return ux;
  }


  private double Shannon (double x)
  {
    if (x > 0.0 && x < 1.0)
       return - x*Math.log(x) - (1.0-x)*Math.log(1.0-x);
    else
       return 0.0;
  }


  private double Yager (Valores valores, int u0, int u1, int t)
  {
    int    i;
    int    total = valores.Total();
    double x, sum=0.0;

    for (i=0; i<total; i++) {
        x = Ux(valores,i,u0,u1,t);
        x = x*(1.0-x);
        sum += x*x;
    }

    return Math.sqrt(sum);
  }


  private double fuzzy (Valores valores, int u0, int u1, int t, int method)
  {
    if (method == FUZZY_ENTROPY) {

       int    i;
       double H = 0;

       for (i=0; i<valores.Total(); i++)
           H += Shannon(Ux(valores,i,u0,u1,t))*h(valores,i);

       return H;

    } else {

       return Yager (valores, u0, u1, t);

    }
  }

  private int FuzzyThresholding (Valores valores, byte method)
  {
    int    total  = valores.Total();
    int    sum;
    int    minsum;
    double S[]    = new double [total];
    double Sbar[] = new double [total];
    double W[]    = new double [total];
    double Wbar[] = new double [total];
    double F[]    = new double [total];
    double suma;
    double maxv   = 0.0;
    double delta;

    int    i, j, start, end, u0, u1;
    int    t, tbest;

    // Histograma: hist -> h(valores,i)

    suma = 0;

    for (i=0; i<total; i++)
        suma += h(valores,i);

    // Histograma acumulado

    S[0] = h(valores,0);
    W[0] = 0;

    for (i=1; i<total; i++){
        S[i] = S[i-1] + h(valores,i);
        W[i] = v(valores,i)*h(valores,i) + W[i-1];
    }

    // Histograma acumulado inverso

    Sbar[total-1] = 0;
    Wbar[total-1] = 0;

    for (i=total-2; i>=0; i--) {
        Sbar[i] = Sbar[i+1] + h(valores,i+1);
        Wbar[i] = Wbar[i+1] + v(valores,i+1)*h(valores,i+1);
    }

    // Umbral

    tbest = -1;

    for (t=1; t<total-1; t++) {

        u0 = n (valores, W[t]/S[t] );
        u1 = n (valores, Wbar[t]/Sbar[t] );

        if (u0!=-1 && u1!=-1) {

           F[t] = fuzzy (valores, u0, u1, t, method) / suma;

           if (F[t] > maxv)
              maxv = F[t];

           if (tbest < 0)
              tbest = t;
           else if (F[t] < F[tbest])
              tbest = t;
        }
    }

    // Encontrar el mejor umbral dentro de un rango

    if (tbest!=-1) {

       delta = F[tbest] + (maxv-F[tbest])*0.05;        /* 5% */

       start = (int)(tbest - delta);

       if (start <= 0)
          start = 1;

       end = (int)(tbest + delta);

       if (end>=total-1)
          end = total-2;

       minsum = 1000000000;

       for (i=start; i<=end; i++) {

           sum = h(valores,i-1) + h(valores,i) + h(valores,i+1);

           if (sum < minsum) {
              t = i;
              minsum = sum;
           }
       }
    } else {

       t = tbest;
    }

    return t;
  }

  // Minimum error thresholding
  // --------------------------

  private double METlog (double x)
  {
    if (x > 0.0)
       return Math.log(x);
    else
       return 0.0;
  }

  private double P1 (Valores valores, int t)
  {
    int    i;
    double sum = 0.0;

    for (i=0; i<=t; i++)
        sum += h(valores,i);

    return sum;
  }

  private double P2 (Valores valores, int t)
  {
    int    i;
    int    total = valores.Total();
    double sum   = 0;

    for (i=t+1; i<total; i++)
        sum += h(valores,i);

    return sum;
  }

  private double u1 (Valores valores, int t)
  {
    int    i;
    double p;
    double sum = 0.0;

    p = P1(valores,t);

    if (p <= 0.0) {

       return 0.0;

    } else {

       for (i=0; i<=t; i++)
           sum += h(valores,i)*v(valores,i);

       return sum/p;
    }

  }

  private double u2 (Valores valores, int t)
  {
    int    i;
    double p;
    double sum = 0.0;

    p = P2(valores,t);

    if (p <= 0.0) {

       return 0.0;

    } else {

       for (i=t+1; i<valores.Total(); i++)
           sum += h(valores,i)*v(valores,i);

       return sum/p;
    }

  }

  private double s1 (Valores valores, int t)
  {
    int    i;
    double p,u,x;
    double sum = 0.0;

    p = P1(valores,t);

    if (p<=0.0) {

       return 0.0;

    } else {

       u = u1(valores,t);

       for (i=0; i<=t; i++) {
           x = (v(valores,i)-u)*(v(valores,i)-u);
           sum += x*h(valores,i);
       }

       return sum/p;
    }
  }

  private double s2 (Valores valores, int t)
  {
    int    i;
    double p,u,x;
    double sum = 0.0;

    p = P2(valores,t);

    if (p<=0.0) {

       return 0.0;

    } else {

       u = u2(valores,t);

       for (i=t+1; i<valores.Total(); i++) {
           x = (v(valores,i)-u)*(v(valores,i)-u);
           sum += x*h(valores,i);
       }

       return sum/p;
    }
  }

  private double J (Valores valores, int t)
  {
    double a = P1(valores,t);
    double b = s1(valores,t);
    double c = P2(valores,t);
    double d = s2(valores,t);

    return 1.0 + 2.0*(a*METlog(b) + c*METlog(d)) - 2.0*(a*METlog(a) + c*METlog(c));
  }

  private int MinimumErrorThresholding (Valores valores)
  {
    int    i,j,t;
    int    total = valores.Total();
    double F, Fbest;

    t     = 0;
    Fbest = J(valores,0);

    for (i=1; i<total-1; i++) {

        F = J(valores,i);

        if (F < Fbest) {
           t     = i;
           Fbest = F;
        }
    }

    return t;
  }

}
