package rf;

// DataConfiguration
// -----------------

/**
    Configuration file format
    -------------------------

    * Similar to MS Windows INI files

    * Sections can be disordered

    Common sections
    ---------------

    [DESCRIPTION]
    Data description (several lines if needed)

    [FORMAT]
    ASCII|IMAGE|JDBC

    [ATTRIBUTES]
    Attribute IDs (one per line)
    IDs corresponding to descriptions, table fields or image files

    [CLASSIFIER]
    Attribute used for classification (optional, except for JDBC format)

    [CLASSES]
    Allowed classes (one per line)


    ASCII format specific sections
    ------------------------------

    [DATA FILE]
    ASCII data file

    [LEARNING FILE]
    File with data used for learning tasks (optional)

    [TEST FILE]
    File with data used for testing


    Image format specific sections
    ------------------------------

    [WIDTH]
    Image width

    [HEIGHT]
    Image height

    [TRAINING FILE]
    Image with labelled pixels

    [LEARNING FILE]
    Image with labelled pixels used for learning

    [TEST FILE]
    Image with labelled pixels used for testing classifiers accuracy


    // JDBC specific sections
    // ----------------------

    [JDBC DRIVER]
    The JDBC Driver used

    [URL]
    The database URL

    [USER]
    User ID

    [PASSWORD]
    User's password

    [TABLE]
    Table ID (the table which contains input data)

*/

import java.sql.*;
import java.util.Vector;

import fbg.util.ConfigFile;
import fbg.util.db.Tabla;

import rf.data.*;


public class DataConfiguration
{
  // Dataset description

  String[] description = null;

  // Attributes

  String attributes[] = null;

  // Classification

  String classifier = null;
  String classes[]  = null;

  // Data format

  public static final byte ASCII_DATA = 1;
  public static final byte JDBC_DATA  = 2;
  public static final byte IMAGE_DATA = 3;

  byte   format = 0;

  // DATA

  Data   data          = null;  // Data container

  String training_file = null;  // Prototype set
  String learning_file = null;  // Learning set
  String test_file     = null;  // Test set

  String path          = "";    // Path to data files

  // JDBC DATA

  String driver   = null;
  String url      = null;
  String user     = null;
  String password = null;
  String tableID  = null;

  // Image DATA

  int    width  = 0;
  int    height = 0;


  // Constructor
  // -----------

  public DataConfiguration (String fileID)
  {
    ConfigFile cfg = new ConfigFile(fileID);

    // Path

    path = cfg.path();

    // Data format

    description = cfg.Section("[DESCRIPTION]");

    format = parseFormatSection ( cfg.Section("[FORMAT]") );

    if ( format == IMAGE_DATA ) {

       width  = parseIntSection ( cfg.Section("[WIDTH]") );
       height = parseIntSection ( cfg.Section("[HEIGHT]") );

    } else if ( format == JDBC_DATA ) {

       driver   = parseStringSection ( cfg.Section("[JDBC DRIVER]") );
       url      = parseStringSection ( cfg.Section("[URL]") );
       user     = parseStringSection ( cfg.Section("[USER]") );
       password = parseStringSection ( cfg.Section("[PASSWORD]") );
       tableID  = parseStringSection ( cfg.Section("[TABLE]") );

    }

    // Data files

    if (  format == ASCII_DATA
       || format == IMAGE_DATA ) {

       training_file = parseStringSection ( cfg.Section("[DATA FILE]") );

       if (training_file==null)
           training_file = parseStringSection ( cfg.Section("[TRAINING FILE]") );

       learning_file = parseStringSection ( cfg.Section("[LEARNING FILE]") );
       test_file     = parseStringSection ( cfg.Section("[TEST FILE]") );
    }

    // Attributes

    attributes = cfg.Section("[ATTRIBUTES]");

    // Classification

    classifier = parseStringSection ( cfg.Section("[CLASSIFIER]") );

    classes    = cfg.Section("[CLASSES]");

    // Data

    data = loadData();
  }

  // Alternative: Get data from an opened JDBC connection

  public DataConfiguration ( String     table, 
                             String     target, 
                             String     fields[], 
                             Connection jdbc)
  {
    // Data format

    description = new String[]{table};

    format      = JDBC_DATA;
    
    // driver   = parseStringSection ( cfg.Section("[JDBC DRIVER]") );
    // url      = parseStringSection ( cfg.Section("[URL]") );
    // user     = parseStringSection ( cfg.Section("[USER]") );
    // password = parseStringSection ( cfg.Section("[PASSWORD]") );

    tableID    = table;
    attributes = fields;

    // Classification
    // --------------

    if (  target == null
       || target.equals("") ) {

       classifier = null;
       classes    = null;

    } else {

       classifier = target;
       classes    = getClasses(jdbc);
    }

    // Data
    // ----

    data = loadTableData(jdbc);
  }

  // Available classes

  public String[] getClasses (Connection connection)
  {
    Vector    result  = new Vector();
    String    array[] = null;
    Statement SQL     = null;
    ResultSet val     = null;
    String    obj;
    int       i;


    try {

        SQL = connection.createStatement();

        val = SQL.executeQuery ( "SELECT DISTINCT " + classifier  
                               + " FROM " + tableID
                               + " ORDER BY " + classifier );

        while (val.next()) {

              obj = val.getString(1);

              if (obj!=null)
                 result.addElement(obj.trim());
        }

        SQL = null;

    } catch (SQLException e) {

        while (e != null) {
              System.out.println(e.getMessage());
              e = e.getNextException();
        }
    }

    array = new String[result.size()];

    for (i=0; i<array.length; i++)
        array[i] = (String) result.elementAt(i);

    result = null;

    return array;
  }

  // Config file parsing
  // -------------------

  // Get a String from section contents

  private String parseStringSection (String array[])
  {
    if (array!=null && array.length>0)
       return array[0];
    else
       return null;
  }

  // Get an integer from section contents

  private int parseIntSection ( String array[] )
  {
    if (array!=null && array.length>0)
       return (new Integer(array[0])).intValue();
    else
       return 0;
  }

  // Get data format:
  // - ASCII
  // - JDBC
  // - Image

  private byte parseFormatSection (String array[])
  {
    String ascii;
    byte   type = 0;

    if (array!=null && array.length>0) {

       ascii = array[0].toUpperCase();

       if (ascii.equals("JDBC")) {

          type = JDBC_DATA;

       } else if (ascii.equals("IMAGE")) {

          type = IMAGE_DATA;

       } else if (ascii.equals("ASCII")) {

          type = ASCII_DATA;

       }
    }

    return type;
  }

  // Load data
  // ---------

  private Data loadData ()
  {
    Data data = null;

    switch (format) {

      case IMAGE_DATA: data = loadImageData();
                       break;

      case JDBC_DATA:  data = loadTableData();
                       break;

      case ASCII_DATA: data = loadAsciiData();
                       break;
    }

    return data;
  }

  // ASCII Format

  private AsciiData loadAsciiData ()
  {
    AsciiData data = new AsciiData(attributes.length);

    if (  learning_file == null
       || test_file == null ) {

       data.setTraining ( path+"/"+training_file, 0.7 );

    } else {

       data.setLearning ( path+"/"+learning_file);

       data.setTest ( path+"/"+test_file);
    }

    return data;
  }

  // Image format

  private ImageData loadImageData ()
  {
    int i;

    ImageData data = new ImageData(width,height);

    for (i=0; i<attributes.length; i++)
        data.addBand(path+"/"+attributes[i]);

    data.setClassification (path+"/"+training_file);
    data.setLearning       (path+"/"+learning_file);
    data.setTest           (path+"/"+test_file);

    return data;
  }

  // JDBC

  private TableData loadTableData ()
  {
    int        i;
    Connection connect = null;
    Tabla      table   = null;
    String     id[]    = null;
    TableData  data    = null;

    // Fields

    if (classifier!=null) {
    
       id = new String[attributes.length+1];

       for (i=0; i<attributes.length; i++)
           id[i] = attributes[i];

       id[attributes.length] = classifier;

    } else {

       id = attributes;
    }

    // JDBC driver

    try {
        Class.forName(driver);
    } catch (Exception error) {
    }

    // JDBC connection

    try {
        connect = DriverManager.getConnection(url,user,password);
    } catch (Exception error) {
    }

    // Table

    table = new Tabla (connect, tableID);

    // Dataset

    data = new TableData (table, id, (classifier!=null) );

    return data;
  }

  private TableData loadTableData (Connection connect)
  {
    int        i;
    Tabla      table   = null;
    String     id[];
    TableData  data    = null;

    // Fields
   
    if (classifier!=null) {
    
       id = new String[attributes.length+1];

       for (i=0; i<attributes.length; i++)
           id[i] = attributes[i];

       id[attributes.length] = classifier;

    } else {

       id = attributes;
    }

    // Table

    table = new Tabla (connect, tableID);

    // Dataset

    data = new TableData (table, id, (classifier!=null) );

    return data;
  }

  // Access to data
  // --------------

  public Data getData ()
  {
    return data;
  }

  public String[] getClasses ()
  {
    return classes;
  }

  public int totalClasses ()
  {
    if (classes!=null)
       return classes.length;
    else
       return 0;
  }

  public String getClass (int n)
  {
    if (classes!=null)
       return classes[n];
    else
       return null;
  }

  public String getClassifier ()
  {
    return classifier;
  }

  public String[] getAttributes ()
  {
    return attributes;
  }

  public int totalAttributes ()
  {
    if (attributes!=null)
       return attributes.length;
    else
       return 0;
  }

  public String getAttribute (int n)
  {
    if (attributes!=null)
       return attributes[n];
    else
       return null;
  }

  public byte getFormat ()
  {
    return format;
  }


  public String[] getDescription ()
  {
    return description;
  }

  public String getDescriptionString ()
  {
    String str = "";
    int    i;

    if (description!=null)
       for (i=0; i<description.length; i++)
           str += description[i]+"\n";

    return str;
  }

  public String getID ()
  {
    if (description!=null)
       return description[0];
    else
       return null;
  }

  // Report
  // ------

  public void print ()
  {
    // Data description

    printStringArray(description);

    // Data format

    System.out.println();

    switch (format) {

      case IMAGE_DATA: System.out.println("Image data "+width+"x"+height);
                       break;

      case JDBC_DATA:  System.out.println("JDBC data");
                       System.out.println("JDBC Driver: "+driver);
                       System.out.println("URL: "+url);
                       System.out.println("User: "+user);
                       System.out.println("Password: "+password);
                       System.out.println("Table: "+tableID);
                       break;

      case ASCII_DATA: System.out.println("ASCII data");
                       break;

      default:         System.out.println("Unknown format");
    }

    // Attributes

    if (attributes!=null) {
       System.out.println();
       System.out.println(attributes.length + " attributes:");
       printStringArray(attributes);
    }

    if (classifier!=null) {
       System.out.println();
       System.out.println("Classifier = "+classifier);
    }

    // Classes

    if (classes!=null) {
       System.out.println();
       System.out.println(classes.length + " classes:");
       printStringArray(classes);
    }

    // Files

    if (  format==ASCII_DATA
       || format==IMAGE_DATA ) {

       if (training_file!=null) {
          System.out.println();
          System.out.println("Training file = "+training_file);
       }

       if (learning_file!=null) {
          System.out.println();
          System.out.println("Learning file = "+learning_file);
       }

       if (test_file!=null) {
          System.out.println();
          System.out.println("Test file = "+test_file);
       }
    }

  }

  private void printStringArray (String array[])
  {
    int i;

    if (array!=null)
       for (i=0; i<array.length; i++)
           System.out.println(array[i]);
  }



  // TEST

  public static void main (String args[])
  {
    // DataConfiguration cfg = new DataConfiguration("igaliko.cfg");
    // DataConfiguration cfg = new DataConfiguration("galaxy.cfg");
    // DataConfiguration cfg = new DataConfiguration("bupa.cfg");
    // DataConfiguration cfg = new DataConfiguration("iono.cfg");
    // DataConfiguration cfg = new DataConfiguration("jdbc.cfg");

    DataConfiguration cfg = new DataConfiguration(args[0]);

    cfg.print();

    System.out.println(cfg.data.instances());

  }

}