package rf.classification.lvq;

// Linear Vector Quantization (LVQ)
// --------------------------------

import rf.classification.Classifier;
import rf.classification.nn.kNNClassifier;

import rf.*;
import rf.data.*;


public class lvq1 extends Classifier
{
  Dataset set;          // Conjunto de datos
  int     N;            // Nmero de patrones
  String  classes[];    // Clases permitidas
  int     Np;           // Nmero de prototipos
  Pattern m[];          // Array de prototipos
  int     r;            // Pasos en el aprendizaje
  float   alpha;        // Razn de aprendizaje

  // Constructor

  public lvq1 ( DataConfiguration config,
                Dataset           data,
                int               np,
                int               steps,
                float             learningRate,
                boolean           useAprioriProbabilities )
  {
    super (config);

    Pattern pattern;
    int     t;

    set     = data;
    N       = set.instances();
    classes = config.getClasses();
    alpha   = learningRate;
    r       = steps;
    Np      = np;
    m       = new Pattern[Np];

    // Inicializacin de los prototipos (etiquetados)

    if (useAprioriProbabilities)
       propInit (set);
    else
       evenInit (set);

    // Proceso iterativo: Aprendizaje adaptativo (correccin)

    for (t=0; t<steps; t++) {

        if (t%N==0)
           set.shuffle();

        Correction(set.pattern(t%N),t);
    }
  }

  // Inicializacin: Valor inicial de los prototipos
  // -----------------------------------------------

  // - Regla 5-NN
  // - NC prototipos por clase

  private void evenInit (Dataset set)
  {
    kNNClassifier knn = new kNNClassifier(cfg,set,5);

    int           Nc     = Np / classes.length;
    int           init[] = new int[classes.length];
    int           n      = 0;
    int           i,j,c;
    Pattern       pattern;


    for (i=0; i<classes.length; i++)
        init[i] = 0;

    n = 0;

    for (i=0; i<N && n<Np; i++) {

        pattern = set.pattern(i);
        c       = classNo(pattern);

        if (knn.classify(pattern)==c && c<noc && init[c]<Nc) {
           m[n] = new Pattern(pattern);
           init[c]++;
           n++;
        }
    }

    if (n<Np)
       Np = n;
  }

  // - Regla 5-NN
  // - Nmero de prototipos por clase segn probabilidades a priori

  private void propInit (Dataset set)
  {
    kNNClassifier knn = new kNNClassifier(cfg,set,5);

    int           Nc[]   = new int[noc];
    int           init[] = new int[noc];
    int           freq[] = new int[noc];
    int           n      = 0;
    int           i,j,c;
    Pattern       pattern;


    for (i=0; i<noc; i++) {
        init[i] = 0;
        freq[i] = 0;
    }

    for (i=0; i<N; i++) {
        c = classNo(set.pattern(i));

        if (c<noc)
           freq[c]++;
    }

    for (i=0; i<noc; i++) {
        // Nc[i] = (freq[i]*Np)/set.instances();
        Nc[i] = 1 + (freq[i]*(Np-noc))/set.instances();
        // System.out.println(Nc[i]);
    }


    n = 0;

    for (i=0; i<N && n<Np; i++) {

        pattern = set.pattern(i);
        c       = classNo(pattern);

        if (c<noc)
           if (knn.classify(pattern)==c && init[c]<Nc[c]) {
               m[n] = new Pattern(pattern);
               init[c]++;
               n++;
               //System.out.println(n);
           }
    }

    if (n<Np)
       Np = n;
  }

  private int classNo (Pattern pattern)
  {
    int    c;
    String label = pattern.label();

    for (c=0; c<noc && !label.equals(classes[c]); c++)
        ;

    return c;
  }

  // Correccin
  // ----------

  private void Correction (Pattern pattern, int t)
  {
    Pattern nearest;
    float   mindist;
    float   dist;
    float   f,v;
    int     i;

    // Prototipo ms cercano a 'pattern'

    nearest = m[0];
    mindist = distance(pattern,m[0]);

    for (i=1; i<Np; i++) {
        dist = distance(pattern,m[i]);

        if (dist<mindist) {
           mindist = dist;
           nearest = m[i];
        }
    }

    f = learningRate(t);

    if (classNo(pattern)==classNo(nearest)) {

       // Premio

       for (i=0; i<noa; i++) {
           v = nearest.attribute (i);
           v = v + f*(pattern.attribute(i)-v);
           nearest.attribute(i,v);
       }

    } else {

       // Castigo

       for (i=0; i<noa; i++) {
           v = nearest.attribute (i);
           v = v - f*(pattern.attribute(i)-v);
           nearest.attribute(i,v);
       }
    }
  }

  // Secuencia montona decreciente de coeficientes escalares

  // LINEAR

  private float learningRate (int t)
  {
    return alpha*(1.0f-(float)t/(float)r);
  }

  // INVERSE
/*
  private float learningRate (int t)
  {
    float C = r/100.0f;

    return C*alpha/(C+t);
  }
*/

  // Clasificacin
  // -------------

  // Distancia entre patrones

  private float distance (Pattern p, Pattern q)
  {
    float diff;
    float d = 0;
    int   i;

    for (i=0; i<noa; i++) {
        diff  = p.attribute(i) - q.attribute(i);
        d    += diff*diff;
    }

    return d;
  }

  // Clasificacin (regla 1-NN)

  public int classify (Pattern p)
  {
    int     i,j,c;
    float   mindist;
    float   dist;
    String  label = null;

    // Vecino ms cercano

    if (Np>0) {
       mindist = distance(p,m[0]);
       label   = m[0].label();

       for (i=1; i<Np; i++) {
           dist = distance(p,m[i]);

           if (dist<mindist) {
              mindist = dist;
              label = m[i].label();
           }
       }
    }

    // Cdigo de la clase

    if (label!=null) {

       for (c=0; c<noc && !label.equals(classes[c]); c++)
           ;

    } else {

       c = noc;
    }

    return c;
  }

  // TEST
  // ----

  public static void main (String args[])
  {
    DataConfiguration cfg;
    Data              data;
    Classifier        classifier;
    int               Np,r;
    float             alpha;
    boolean           useApriori = false;


    if (args.length!=4 && args.length!=5) {

       System.out.println ("Clasificador LVQ1");
       System.out.println ();
       System.out.println ("Sintaxis: lvq1 <fichero_cfg> <Np> <r> <alpha> [apriori]");
       System.out.println ();
       System.out.println ("Indicaciones:");
       System.out.println ("  alpha < 0.1  (vg: 0.02, 0.03...)");
       System.out.println ("  50 Np <= r <= 200 Np");

    } else {

       cfg   = new DataConfiguration (args[0]);
       Np    = (new Integer(args[1])).intValue();
       r     = (new Integer(args[2])).intValue();
       alpha = (new Float(args[3])).floatValue();

       useApriori = (args.length==5);

       if (cfg.totalClasses()<2 || cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (Np<cfg.totalClasses())
          Np = cfg.totalClasses();

       System.out.print ( cfg.getDescriptionString() );

       data = cfg.getData();

       System.out.println ();
       System.out.println ("- Clases: "+cfg.totalClasses());
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+data.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+data.testInstances()+" patrones");
       System.out.println ();

       System.out.println ( "Clasificador LVQ1");
       System.out.println ( "- " + Np + " prototipos");
       System.out.println ( "- " + r + " pasos de aprendizaje");
       System.out.println ( "- alpha(0) = "+alpha);

       if (useApriori) {
          System.out.println ("- PROPINIT");
       } else {
          System.out.println ("- EVENINIT");
       }

       System.out.println();

       classifier = new lvq1 (cfg, data.learningSet(), Np, r, alpha, useApriori);

       System.out.println("CONJUNTO DE APRENDIZAJE");

       classifier.printContingencyTable ( System.out, classifier.evaluation(data.learningSet()) );

       System.out.println("CONJUNTO DE PRUEBA");

       classifier.printContingencyTable ( System.out, classifier.evaluation(data.testSet()) );
    }
  }
}
