package rf.classification.parametric;

// Clasificador paramtrico lineal: Distancia de Mahalanobis
// ---------------------------------------------------------
//  Todas las variables con la misma matriz de covarianza

import rf.classification.Classifier;

import rf.*;
import rf.data.*;

import fbg.util.math.Matrix;


public class MahalanobisClassifier extends Classifier
{
  boolean useAprioriProbabilities;

  float   prob[];     // Instancias de cada clase
  float   mean[][];   // Media de un atributo para los elementos de una clase
  Matrix  covariance; // Matriz de covarianza de los datos
  Matrix  invcov;     // Inversa de la matriz de covarianza

  // Constructor

  public MahalanobisClassifier ( DataConfiguration config,
                                 Dataset           data )
  {
    this (config, data, false);
  }

  public MahalanobisClassifier ( DataConfiguration config,
                                 Dataset           set,
                                 boolean           apriori )
  {
    super(config);

    useAprioriProbabilities = apriori;

    prob = new float[noc];
    mean = new float[noa][noc];

    String     classes[] = cfg.getClasses();
    Statistics stats     = new Statistics (set);
    String     label;
    int        N         = set.instances();
    int        i,k,c;
    float      freq[]    = new float[noc];
    float      dif;

    // Medias de cada variable para cada clase

    for (i=0; i<noc; i++)
        freq[i] = 0;

    for (k=0; k<N; k++) {

        label = set.label (k);

        for (c=0; c<noc && !label.equals(classes[c]); c++)
            ;

        if (c<noc) {

           freq[c]++;

           for (i=0; i<noa; i++)
               mean[i][c] += set.value(k,i);
        }
    }

    for (c=0; c<noc; c++) {

        if (freq[c]>0) {
           for (i=0; i<noa; i++)
               mean[i][c] /= freq[c];
        }

        prob[c] = freq[c] / N;
    }

    // Matriz de covarianza

    covariance = stats.covariance();

    invcov     = covariance.inverse();

    if (invcov==null)
       throw ( new ArithmeticException ("Singular covariance matrix") );

    // Standard output
    // printStats();
  }

  private void printStats ()
  {
    int i,j,c;

    for (c=0; c<noc; c++) {

        for (i=0; i<noa; i++)
            System.out.print("\t"+mean[i][c]);

        System.out.println();
    }

    for (i=0; i<noa; i++) {

        for (j=0; j<noa; j++)
            System.out.print("\t"+covariance.getCell(i,j));

        System.out.println();
    }
  }

  // Funcin discriminante

  private float discriminant (int c, Pattern p)
  {
    Matrix diff;
    Matrix difft;
    Matrix mahalanobis;
    float  dist;
    float  g = 0;
    int    i;

    if (prob[c]>0) {

       diff = new Matrix (noa, 1);

       for (i=0; i<noa; i++)
           diff.setCell (i, 0, p.attribute(i) - mean[i][c]);

       difft = diff.transpose();

       mahalanobis = difft.mult ( invcov.mult ( diff ) );

       dist = (float) mahalanobis.getCell(0,0);

       if (!useAprioriProbabilities)
          g = dist;
       else
          g = dist/2 - (float) Math.log((double)prob[c]);

    } else {

       g = 1e30f;
    }

    return g;
  }

  // Clasificador

  public int classify (Pattern p)
  {
    float min   = discriminant (0,p);
    float g;
    int   label = 0;
    int   i;

    for (i=1; i<noc; i++) {

        g = discriminant (i,p);

        if (g<min) {
           min   = g;
           label = i;
        }
    }

    return label;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration     c  = null;
    Data                  d  = null;
    MahalanobisClassifier lc = null;

    if (args.length<1 || args.length>2) {

       System.out.println ("Clasificador lineal tipo 2 (distancia de Mahalanobis)");
       System.out.println ();
       System.out.println ("Sintaxis: MahalanobisClassifier <fichero_cfg> [apriori]");

    } else {

       c  = new DataConfiguration (args[0]);

       if (c.totalClasses()<2 || c.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( c.getDescriptionString() );

       d  = c.getData();

       System.out.println ();
       System.out.println ("- Clases: "+c.totalClasses());
       System.out.println ("- Atributos: "+c.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+d.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+d.testInstances()+" patrones");
       System.out.println ();

       try {

            lc = new MahalanobisClassifier ( c, d.learningSet(), (args.length==2) );

       } catch (ArithmeticException e) {

            System.err.println("Matriz de covarianza singular\n");
       }

       if (lc!=null)
          lc.printContingencyTable ( lc.evaluation(d.testSet()) );
    }
  }
}


