package rf.clustering;

// Algoritmo de mxima distancia (Batchelor & Wilkins

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Clustering with an unknown number of classes:
 * Batchelor and Wilkins' Algorithm
 */

public class BatchelorWilkins extends ClusteringAlgorithm
{
  // Constructor

  public BatchelorWilkins (DataConfiguration cfg, Dataset set, Distance dm, float f)
  {
    super (cfg, set, dm);

    int     i,j,k;
    int     A, Aprev;
    int     N = set.instances();
    Pattern p;
    Cluster c;
    Cluster asignacion[] = new Cluster[N];
    int     best;
    float   dist,maxdist,mindist,meandist;

    // Primer cluster: Patrn escogido al azar

    i = (int)(N*Math.random());
    c = addCluster();
    c.assign ( set.pattern(i) );
    asignacion[i] = c;

    // Segundo cluster: Patrn ms alejado del primer cluster

    maxdist = -1;
    best    = -1;

    for (i=0; i<N; i++) {
        dist = metrics.distance ( c, set.pattern(i) );

        if (dist>maxdist) {
           maxdist = dist;
           best = i;
        }
    }

    c = addCluster();
    c.assign ( set.pattern(best) );
    asignacion[best] = c;

    // Bucle principal

    A = 2;

    do {

       Aprev = A;

       // VERBOSE: System.out.println(A);

       // Mximo de las distancias mnimas a los centroides

       maxdist = -1;

       for (i=0; i<N; i++) {
           if (asignacion[i]==null) {

              p       = set.pattern(i);
              mindist = metrics.distance(cluster(0),p);

              for (k=1; k<A; k++) {
                  dist = metrics.distance(cluster(k),p);

                  if (dist<mindist) {
                     mindist = dist;
                  }
              }

              if (mindist>maxdist) {
                 maxdist = mindist;
                 best    = i;
              }
           }
       }

       // Distancia media entre agrupamientos

       meandist = 0;

       for (i=0; i<A; i++)
           for (j=i+1; j<A; j++)
               meandist += metrics.distance(cluster(i),cluster(j));

       meandist /= (A*(A-1)/2);


       //  Crear nuevo agrupamiento ?

       if (maxdist>f*meandist) {
          p = set.pattern(best);
          c = addCluster();

          c.assign (p);
          asignacion[best] = c;

          A++;
       }

    } while (A!=Aprev && A<N);

    // Asignar cada patrn al agrupamiento ms cercano

    for (i=0; i<N; i++) {
        if (asignacion[i]==null) {

           p       = set.pattern(i);
           best    = 0;
           mindist = metrics.distance(cluster(0),p);

           for (k=1; k<A; k++) {
               dist = metrics.distance(cluster(k),p);

               if (dist<mindist) {
                  mindist = dist;
                  best    = k;
               }
           }

           cluster(best).assign (p);
           asignacion[i] = cluster(best);
        }
    }
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    float               f;
    ClusteringAlgorithm clustering;


    if (args.length!=2) {

       System.out.println ("Sintaxis: BatchelorWilkins <fichero_cfg> <f>");

    } else {

       cfg = new DataConfiguration (args[0]);
       f   = (new Float(args[1])).floatValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("Batchelor & Wilkins\n");

       clustering = new BatchelorWilkins (cfg,set,dist,f);

       clustering.printReport();
    }
  }
}