package rf.clustering;

// CLUSTER

import java.util.Vector;

import rf.data.*;
import rf.distance.*;


public class Cluster
{
  float  centroide[];   // Centroide del cluster
  float  varianza[];    // Varianza del agrupamiento
  int    clase[];       // Distribucin de clases en el cluster

  int    noa;           // Nmero de atributos
  int    noc;           // Nmero de clases

  int    casos;         // Nmero de patrones incluidos en el cluster

  String id[];          // IDs de las distintas clases


  float    dispersion;   // Dispersin: Distancia media al centroide
  Distance dm;
  boolean  valid;
  Vector   patterns;

  // Constructor

  public Cluster (Distance metrics, int attributes, String classes[])
  {
    noa = attributes;

    id  = classes;

    if (classes!=null)
       noc = classes.length;
    else
       noc = 0;

    casos = 0;

    centroide = new float[noa];
    varianza  = new float[noa];
    clase     = new int[noc];

    // Inicializacin

    int i;

    for (i=0; i<noa; i++) {
        centroide[i] = 0;
        varianza[i]  = 0;
    }

    for (i=0; i<noc; i++)
        clase[i] = 0;

    valid    = false;
    patterns = new Vector();
    dm       = metrics;
  }

  // Asignacin de un patrn a un cluster

  public void assign ( Pattern p )
  {
    int    i;
    float  diff1,diff2;
    float  avg1,avg2;
    String label;

    for (i=0; i<noa; i++) {

        avg1 = centroide[i];
        avg2 = p.attribute(i);

        centroide[i] = (casos*avg1+avg2)/(casos+1);

        diff1 = avg1-centroide[i];
        diff2 = avg2-centroide[i];

        varianza[i] = (casos*(varianza[i]+diff1*diff1)+diff2*diff2)/(casos+1);
    }

    if (noc>0) {

       label = p.label();

       for (i=0; i<noc && !id[i].equals(label); i++)
           ;

       if (i<noc)
          clase[i]++;
    }

    patterns.addElement(p);
    valid = false;

    casos ++;
  }


  public void deassign (Pattern p)
  {
    int    i;
    float  n,n2;
    float  avg,avg1,avg2;
    float  dif1,dif2;
    String label;

    for (i=0; i<noa; i++) {

        n    = casos;
        n2   = casos-1;

        avg  = centroide[i];
        avg1 = p.attribute(i);
        avg2 = (n*avg-avg1)/n2;

        dif1 = avg1-avg;
        dif2 = avg2-avg;

        centroide[i] = avg2;
        varianza[i]  = (n*varianza[i]-dif1*dif1)/n2-dif2*dif2;
    }

    if (noc>0) {

       label = p.label();

       for (i=0; i<noc && !id[i].equals(label); i++)
           ;

       if (i<noc)
          clase[i]--;
    }

    patterns.removeElement(p);
    valid = false;

    casos --;
  }

  // Vaciado del agrupamiento

  public void clear ()
  {
    int i;

    for (i=0; i<noa; i++) {
        centroide[i] = 0;
        varianza[i]  = 0;
    }

    for (i=0; i<noc; i++)
        clase[i] = 0;

    patterns = new Vector();
    valid    = false;
    casos    = 0;
  }

  // Fusin de agrupamientos

  public void add ( Cluster c )
  {
    int   i;
    int   total;
    int   n1,n2;
    float avg1,avg2;
    float var1,var2;
    float dif1,dif2;

    n1    = casos;
    n2    = c.casos;
    total = n1+n2;

    for (i=0; i<noa; i++) {

        avg1 = centroide[i];
        avg2 = c.centroide[i];
        var1 = varianza[i];
        var2 = c.varianza[i];

        // Media en composicin de poblaciones:
        // Media de las medias

        centroide[i] = (n1*avg1 + n2*avg2)/total;

        // Varianza en composicin de poblaciones:
        // Media ponderada de las varianzas + Varianza de las medias

        dif1 = avg1-centroide[i];
        dif2 = avg2-centroide[i];

        varianza[i] = (n1*var1 + n2*var2)/total
                    + (n1*dif1*dif1+n2*dif2*dif2)/total;
    }

    for (i=0; i<noc; i++)
        clase[i] += c.clase[i];


    for (i=0; i<c.casos; i++)
        patterns.addElement (c.patterns.elementAt(i));

    valid = false;

    casos = total;
  }

  // Patrones incluidos en el cluster
  
  public int size()
  {
    return casos;
  }

  // Centroide

  public float centroid (int n)
  {
    return centroide[n];
  }

  // Varianza

  public float variance (int n)
  {
    return varianza[n];
  }

  // Desviacin tpica

  public float deviation (int n)
  {
    return (float)Math.sqrt(varianza[n]);
  }

  // Dispersin: Distancia media al centroide

  public float dispersion ()
  {
    int     i;

    if (!valid) {

       dispersion = 0;

       for (i=0; i<casos; i++)
           dispersion += dm.distance(this, (Pattern) patterns.elementAt(i) );

       dispersion /= casos;

       valid = true;
    }

    return dispersion;
  }


  // Frecuencia de cada clase dentro del agrupamiento

  public int freq (int c)
  {
    return clase[c];
  }

  // Clase ms frecuente dentro del cluster (-1 si no es aplicable)

  public int classID ()
  {
    int i;
    int best  = -1;
    int fbest =  0;
    int casos;

    for (i=0; i<noc; i++) {
        if (clase[i]>fbest) {
           best  = i;
           fbest = clase[i];
        }
    }

    return best;
  }

  public String label ()
  {
    int best = classID();

    if (best!=-1)
       return id[best];
    else
       return null;
  }

  // Patrn asociado al centroide del cluster

  public Pattern pattern ()
  {
    return new Pattern ( centroide, label() );
  }

  // Salida estndar

  public String toString ()
  {
    int    i;
    String str = casos + " patrones\nMEDIA = [";

    for (i=0; i<noa-1; i++)
        str += centroide[i]+",";

    str += centroide[noa-1]+"]\nVARIANZA = [";

    for (i=0; i<noa-1; i++)
        str += varianza[i]+",";

    str += varianza[noa-1]+"]\n";

    if (noc>0) {
       str += "CLASES = [";

       for (i=0; i<noc-1; i++)
           str += clase[i]+",";

       str += clase[noc-1]+"]\n";
       str += "Clase "+label()+"\n";
    }

    return str;
  }
}