package rf.clustering;

// GRASP: Greedy Randomized Adaptive Search Procedure
// - Bsqueda local = K-MEDIAS
// - Mutacin uniforme (30%)
// - #RCL = 0.05N

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Clustering with a known number of classes:
 * Minimization of sum of squared distance (GRASP + K-MEANS)
 */

public class GRASPClustering extends ClusteringAlgorithm
{
  Dataset set;
  int     N;
  int     K;

  // Constructor

  public GRASPClustering ( DataConfiguration cfg, Dataset data, Distance dm,
                           int k, int I, int C )
  {
    super (cfg,data,dm);

    set = data;
    N   = set.instances();
    K   = k;

    int   i,j;
    int   rcl = (int) (0.05*N);
    int   local[]  = null;
    int   actual[] = null;
    int   mejor[]  = new int[N];
    float coste_mejor = 1e30f;
    float coste;

    // Inicializacin

    if (K>N)
       K = N;

    if (rcl<2)
       rcl = 2;

    for (j=0; j<K; j++)
        addCluster();

    // Bucle GRASP

    for (i=0; i<I; i++) {

        // Solucin greedy aleatoria

        actual = randomizedGreedy (rcl);

        // Ciclo interno

        for (j=0; j<C; j++) {

            // Bsqueda local

            local = localSearch (actual);

            coste = Coste(local);

            // Actualizacin de la mejor solucin

            if (coste<coste_mejor) {
               coste_mejor = coste;
               mejor = local;
            }

            // Mutacin

            if (j<C-1)
               actual = mutation(local);
        }
    }

    // Mejor asignacin = mejor

    assign(mejor);

    // Eliminacin de clusters vacos

    for (j=K-1; j>=0; j--)
        if (cluster(j).size()==0)
           removeCluster(j);

    K = clusters();
  }


  // Asignacin de cada patrn al cluster especificado

  private void assign (int actual[])
  {
    int i,j;

    for (j=0; j<K; j++)
        cluster(j).clear();

    for (i=0; i<N; i++)
        cluster(actual[i]).assign (set.pattern(i));
  }


  // Coste de una asignacin

  private float Coste (int asignacion[])
  {
    int   i;
    float c = 0;

    assign(asignacion);

    for (i=0; i<N; i++)
        c += metrics.distance ( cluster(asignacion[i]), set.pattern(i) );

    return c;
  }


  // Bsqueda local: K-MEANS

  private int[] localSearch (int inicial[])
  {
    int     i,j;
    Pattern p;
    int     actual[] = new int[N];
    int     cambios;
    float   dist,mindist;
    int     best;

    // Inicializacin

    assign(inicial);

    // Bucle principal

    do {

        cambios = 0;

        // Asignacin al cluster ms cercano

        for ( i=0; i<N; i++) {

            p = set.pattern(i);

            best    = 0;
            mindist = metrics.distance(cluster(0),p);

            for (j=1; j<K; j++) {

                dist = metrics.distance(cluster(j),p);

                if (dist<mindist) {
                   best = j;
                   mindist = dist;
                }
            }

            if (best!=actual[i]) {
               actual[i] = best;
               cambios++;
            }
        }

        // Actualizacin de los centroides de los clusters

        if (cambios>0)
           assign(actual);

    } while (cambios>0);

    return actual;
  }


  // Algoritmo GREEDY aleatorio
  // --------------------------
  // Construccin de una asignacin greedy aleatoria
  // Lista RCL = 'size' patrones ms alejados de los centroides

  private int[] randomizedGreedy (int size)
  {
    int     solucion[] = new int[N];

    int     maxdists;
    float   maxdist[] = new float[size];
    Pattern mejor[]   = new Pattern[size];

    Pattern caso;
    Pattern centroide;
    float   mindist,dist;
    int     asignacion;
    int     i,k,p,aleatorio;

    // Inicializacin de los clusters

    for (i=0; i<K; i++)
        cluster(i).clear();

    // Se escoge aleatoriamente el centro del primer cluster

    aleatorio = (int)(N*Math.random());

    cluster(0).assign(set.pattern(aleatorio));

    // Se escogen los centroides de los dems clusters

    for (k=1; k<K; k++) {

        // Se buscan los patrones ms alejados
        // del conjunto de centroides ya escogido

        maxdists = 0;

        for (p=0; p<N; p++) {

            caso = set.pattern(p);

            mindist = metrics.distance(cluster(0),caso);

            for (i=1; i<k; i++) {

                dist = metrics.distance(cluster(i),caso);

                if (dist<mindist)
                   mindist = dist;
            }

            // Vector de casos ms alejados a los centroides

            if (maxdists<size) {

               maxdist[maxdists] = mindist;
               mejor[maxdists] = caso;
               maxdists++;

               for (i=maxdists-2; i>=0 && maxdist[i]<maxdist[i+1]; i--) {
                   caso = mejor[i];
                   dist = maxdist[i];
                   mejor[i] = mejor[i+1];
                   maxdist[i] = maxdist[i+1];
                   mejor[i+1] = caso;
                   maxdist[i+1] = dist;
               }

            } else {

               if (mindist>maxdist[size-1]) {
                  maxdist[size-1] = mindist;
                  mejor[size-1] = caso;

                  for (i=size-2; i>=0 && maxdist[i]<maxdist[i+1]; i--) {
                      caso = mejor[i];
                      dist = maxdist[i];
                      mejor[i] = mejor[i+1];
                      maxdist[i] = maxdist[i+1];
                      mejor[i+1] = caso;
                      maxdist[i+1] = dist;
                  }
               }
            }
        }

        // Uno de los casos ms lejanos se aade al conjunto de centroides

        aleatorio = (int)(maxdists*Math.random());

	centroide = mejor[aleatorio];

	cluster(k).assign(centroide);
    }

    // Se realiza la asignacin al cluster ms cercano
    // a partir de los centroides escogidos

    for (i=0; i<N; i++) {

        caso = set.pattern(i);

        mindist    = metrics.distance(cluster(0),caso);
        asignacion = 0;

        for (k=1; k<K; k++) {
            dist = metrics.distance(cluster(k),caso);

            if (dist<mindist) {
               mindist = dist;
               asignacion = k;
            }
        }

        solucion[i] = asignacion;
    }

    // Resultado

    return solucion;
  }

  // Operador de mutacin ("in place")
  // ---------------------------------
  // Alterar cada componente del vector asignacin con una probabilidad 0.3

  private int[] mutation (int asignacion[])
  {
    int i;
    int cambios = 0;

    do {

       for (i=0; i<N; i++) {

           if (Math.random()<0.3) {
              asignacion[i] = (int)(K*Math.random());
              cambios++;
           }
       }

    } while (cambios==0);

    return asignacion;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    int                 k,i,c;
    ClusteringAlgorithm clustering;


    if (args.length!=4) {

       System.out.println ("Sintaxis: GRASPClustering <fichero_cfg> <k> <I> <C>");

    } else {

       cfg = new DataConfiguration (args[0]);
       k   = (new Integer(args[1])).intValue();
       i   = (new Integer(args[2])).intValue();
       c   = (new Integer(args[3])).intValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (k<2)
          k = 2;

       if (i<1)
          i = 1;

       if (c<1)
          c = 1;

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("GRASP + K-Medias\n");

       clustering = new GRASPClustering (cfg,set,dist,k,i,c);

       clustering.printReport();
    }
  }
}
