package rf.clustering;

// Algoritmo ISODATA

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Clustering with a known number of classes:
 * ISODATA [Iterative Self-Organizing Data Analysis Techniques]
 */

// - Seleccionar configuracin inicial de los clusters
//    - Escoger 'nc' patrones como centros de 'nc' agrupamientos
//    - Asignar cada patrn al agrupamiento ms cercano
//
// - Eliminar clusters que no contengan suficientes patrones (tn)
//
// - Dividir clusters con desviacin estndar mayor que 's' siempre
//   que la distancia de los patrones al centroide del cluster est
//   por encima de la media.
//
// - Fusionar los pares de clusters tales que la distancia entre
//   sus centroides est por debajo de 'c'.


public class ISODATA extends ClusteringAlgorithm
{
  int A;   // Nmero actual de agrupamientos == clusters()

  // Constructor
  // -----------

  // K  -> Nmero deseado de agrupamientos
  // nc -> Nmero inicial de clusters
  // tn -> Nmero mnimo de patrones en un agrupamiento
  // s  -> Dispersin mxima permitida en un cluster
  // c  -> Distancia mnima entre clusters
  // L  -> Nmero de de pares de agrupamientos que pueden mezclarse en una iteracin
  // I  -> Mximo nmero de iteraciones permitidas

  public ISODATA ( DataConfiguration cfg, Dataset set, Distance dm,
                   int K, int nc, int tn, float s, float c, int L, int I )
  {
    super (cfg,set,dm);

    int     i,j;
    int     iter;
    int     N = set.instances();           // Nmero de patrones
    Cluster actual[] = new Cluster[N];     // Asignacin actual de clusters
    float   dispersion,dev,maxdev;

    // Comprobaciones iniciales

    if (nc>N)
       nc = N;

    if (K>N)
       K = N;

    // Seleccin inicial de los centros de 'nc' agrupamientos

    for (j=0; j<nc; j++) {
        addCluster();

        do {
           i = (int)(N*Math.random());
        } while (actual[i]!=null);

        cluster(j).assign ( set.pattern(i) );
        actual[i] = cluster(j);
    }

    A = nc;

    // Bucle principal

    for (iter=1; iter<=I; iter++) {

        reassign(set,actual);

        // Eliminar agrupamientos con pocos patrones (<tn)

        for (j=A-1; j>=0; j--)
            if (cluster(j).size()<tn) {
               removeCluster(j);
               A--;
               // reassign (set,actual);
            }

        // Dispersin media de los agrupamientos

        dispersion = 0;

        for (j=0; j<A; j++)
            dispersion += cluster(j).size() * cluster(j).dispersion();

        dispersion /= N;


        // Reajuste de los agrupamientos

        if (iter>=I) {

           lumping (0,L);  // LTIMA ITERACIN: c=0

        } else {

           if ( (A>K/2 && iter%2==0) || (A>=2*K) ) {

              lumping (c,L);

           } else {

              division ( set.attributes(), s, dispersion, tn, K);
           }
        }

        // Reajustar parmetros

        // ... [opcional]
    }

    reassign(set,actual);
  }

  // Actualizacin de los agrupamientos

  private int reassign (Dataset set, Cluster actual[])
  {
    int     i,j;
    Pattern p;
    int     N = set.instances();
    Cluster best;
    float   dist,mindist;
    int     cambios = 0;

    // Asignacin al cluster ms cercano

    for ( i=0; i<N; i++) {

        p = set.pattern(i);

        best    = null;
        mindist = 1e30f;

        for (j=0; j<A; j++) {

            dist = metrics.distance(cluster(j),p);

            if (dist<mindist) {
               best = cluster(j);
               mindist = dist;
            }
        }

        if (best!=actual[i]) {
           actual[i] = best;
           cambios++;
        }
    }

    if (cambios>0) {

       // Actualizacin de los agrupamientos

       for (j=0; j<A; j++)
           cluster(j).clear();

       for (i=0; i<N; i++)
           if (actual[i]!=null)
              actual[i].assign (set.pattern(i));
    }

    return cambios;
  }


  // Divisin de agrupamientos

  private void division (int noa, float s, float dispersion, int tn, int K)
  {
    int   i,j;
    float dev,maxdev;
    int   imax;
    int   Nc = A;

    for (j=Nc-1; j>=0; j--) {

        maxdev = cluster(j).deviation(0);
        imax   = 0;

        for (i=1; i<noa; i++) {
            dev = cluster(j).deviation(i);

            if (dev>maxdev) {
               maxdev = dev;
               imax = i;
            }
        }

        if (maxdev>s) {

           if (  (cluster(j).dispersion()>dispersion && cluster(j).size()>2*(tn+1))
              || (A<=K/2) ) {

              split(j,imax);
              A++;
           }
        }
    }
  }

  // Divide un cluster en dos (cluster n, atributo i)

  private void split (int n, int i)
  {
    float   dattr  = 0.6f * cluster(n).deviation(i);
    Pattern z      = cluster(n).pattern();
    int     a;
    float   attr[] = new float[z.attributes()];
    Pattern z1,z2;
    Cluster cluster;

    removeCluster(n);

    // z+

    for (a=0; a<z.attributes(); a++)
        attr[a] = z.attribute(a);

    attr[i] += dattr;

    z1 = new Pattern(attr,z.label());

    cluster = addCluster();
    cluster.assign(z1);

    // z-

    for (a=0; a<z.attributes(); a++)
        attr[a] = z.attribute(a);

    attr[i] -= dattr;

    z2 = new Pattern(attr,z.label());

    cluster = addCluster();
    cluster.assign(z2);
  }



  // Fusin de agrupamientos

  private void lumping (float c, int L)
  {
    int     i,j,n,d;
    int     pairs     = A*(A-1)/2;
    int     di[]      = new int[pairs];
    int     dj[]      = new int[pairs];
    float   dist[]    = new float[pairs];
    boolean used[]    = new boolean[A];
    int     cluster[] = new int[A];
    int     mezclas;
    int     itmp,jtmp;
    float   dtmp;

    // Distancias entre pares de clusters

    d = 0;

    for (i=0; i<A; i++) {
        for (j=i+1; j<A; j++) {
            di[d]   = i;
            dj[d]   = j;
            dist[d] = metrics.distance (cluster(i), cluster(j));

            for (n=d-1; n>=0 && dist[n]>dist[n+1]; n--) { // Ordenacin
                itmp = di[n];
                jtmp = dj[n];
                dtmp = dist[n];
                di[n] = di[n+1];
                dj[n] = dj[n+1];
                dist[n] = dist[n+1];
                di[n+1] = itmp;
                dj[n+1] = jtmp;
                dist[n+1] = dtmp;
            }

            d++;
        }
    }

    // Fusiones (L como mximo)

    for (i=0; i<A; i++) {
        cluster[i] = i;
        used[i] = false;
    }

    if (pairs<L)
       L = pairs;

    for (i=0; i<L && dist[i]<c; i++) {
        if (!used[di[i]] && !used[dj[i]]) {

           mezcla(cluster[di[i]],cluster[dj[i]]);

           used[di[i]] = true;
           used[dj[i]] = true;

           for (j=dj[i]+1; j<A; j++)
               cluster[j]--;

           A--;
        }
    }
  }

  // Mezcla de clusters: cluster(i) = cluster(i) U cluster(j)

  private void mezcla (int i, int j)
  {
    cluster(i).add(cluster(j));

    removeCluster(j);
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    int                 k,nc,tn,l,i;
    float               s,c;
    ClusteringAlgorithm clustering;


    if (args.length!=8) {

       System.out.println ("Sintaxis: ISODATA <fichero_cfg> <k> <nc> <tn> <s> <c> <l> <i>");

    } else {

       cfg = new DataConfiguration (args[0]);
       k   = (new Integer(args[1])).intValue();
       nc  = (new Integer(args[2])).intValue();
       tn  = (new Integer(args[3])).intValue();
       s   = (new Float(args[4])).floatValue();
       c   = (new Float(args[5])).floatValue();
       l   = (new Integer(args[6])).intValue();
       i   = (new Integer(args[7])).intValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (k<2)
          k = 2;

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("Algoritmo ISODATA\n");

       clustering = new ISODATA (cfg,set,dist,k,nc,tn,s,c,l,i);

       clustering.printReport();
    }
  }

}