package rf.clustering;

// Mtodos basados en grafos: Matriz de similaridad

import java.util.BitSet;

import rf.*;
import rf.data.*;
import rf.distance.*;

/**
 * Graph Theoretic Clustering Methods:
 * Similarity Matrix
 */

public class SimilarityMatrix extends ClusteringAlgorithm
{
  // Constructor

  public SimilarityMatrix ( DataConfiguration cfg, Dataset set,
                            Distance dist, float theta)
  {
    super (cfg, set, dist);

    BitSet matrix[] = init(set,dist,theta);

    int     i,dim;
    int     p,pmax;
    int     v,vmax;
    int     patterns = set.instances();
    int     asignacion[] = new int[patterns];
    BitSet  cluster;
    Cluster newCluster;


    for (i=0; i<patterns; i++)
        asignacion[i] = i;


    while (matrix.length>0) {

          dim = matrix.length;

          // DEBUG (patrones por agrupar): System.out.println(dim);

          // Fila con ms unos

          pmax = 0;
          vmax = ones(matrix[0],dim);

          for (p=1; p<dim; p++) {

              v = ones(matrix[p],dim);

              if (v>vmax) {
                 vmax = v;
                 pmax = p;
              }
          }

          // Transitividad

          cluster = closure(matrix[pmax],matrix,dim);

          // Aadir cluster

          newCluster = addCluster();

          for (p=0; p<dim; p++)
              if (cluster.get(p))
                 newCluster.assign ( set.pattern(asignacion[p]) );

          // Reduccin de la matriz de similaridad

          for (p=0; p<dim; p++)
              if (cluster.get(p))
                 for (i=p; i<dim-1; i++)
                     asignacion[i] = asignacion[i+1];

          matrix = reduce(matrix,cluster,dim);
    }
  }

  // Inicializacin de la matriz de similaridad (simtrica)

  BitSet[] init (Dataset set, Distance dm, float threshold)
  {
    int     i,j;
    int     size    = set.instances();
    Pattern pi,pj;
    BitSet  array[] = new BitSet[size];

    for (i=0; i<size; i++)
        array[i] = new BitSet(size);

    for (i=0; i<size; i++) {

        array[i].set(i);

        pi = set.pattern(i);

        for (j=i+1; j<size; j++) {

            pj = set.pattern(j);

            if (dm.distance(pi,pj)<=threshold) {
               array[i].set(j);
               array[j].set(i);
            } else {
               array[i].clear(j);
               array[j].clear(i);
            }
        }
    }

    return array;
  }

  // Nmero de unos en un BitSet

  int ones (BitSet vector, int size)
  {
    int i;
    int total = 0;

    for (i=0; i<size; i++)
        if (vector.get(i))
           total++;

    return total;
  }

  // Propiedad de transitividad

  BitSet closure (BitSet vector, BitSet matrix[], int size)
  {
    int    i;
    BitSet result = new BitSet(size);
    BitSet prev;

    result.or(vector);

    do {

       prev = (BitSet) result.clone();

       for (i=0; i<size; i++)
           if (result.get(i))
              result.or(matrix[i]);

    } while (!result.equals(prev));

    return result;
  }

  // Reduccin de la matriz de similaridad

  BitSet[] reduce (BitSet matrix[], BitSet cluster, int size)
  {
    int    i,j,p;
    int    dim     = size - ones(cluster,size);
    BitSet array[] = new BitSet[dim];
    int    index[] = new int[dim];

    for (i=0; i<dim; i++)
        array[i] = new BitSet(dim);

    p = 0;

    for (i=0; i<size; i++)
        if (!cluster.get(i)) {
           index[p] = i;
           p++;
        }

    for (i=0; i<dim; i++)
        for (j=0; j<dim; j++)
            if (matrix[index[i]].get(index[j]))
               array[i].set(j);
            else
               array[i].clear(j);

    return array;
  }

  // TEST

  public static void main (String args[])
  {
    DataConfiguration   cfg;
    Data                dat;
    Dataset             set;
    Distance            dist;
    float               theta;
    ClusteringAlgorithm clustering;


    if (args.length!=2) {

       System.out.println ("Sintaxis: SimilarityMatrix <fichero_cfg> <theta>");

    } else {

       cfg   = new DataConfiguration (args[0]);
       theta = (new Float(args[1])).floatValue();

       if (cfg.totalAttributes()==0) {
          System.out.println ("Fichero CFG incorrecto");
          System.exit(1);
       }

       if (theta<0) {
          System.out.println ("<theta> debe ser positiva");
          System.exit(2);
       }

       System.out.print ( cfg.getDescriptionString() );

       dat = cfg.getData();

       System.out.println ();
       System.out.println ("- Atributos: "+cfg.totalAttributes());
       System.out.println ("- Conjunto de entrenamiento: "+dat.learningInstances()+" patrones");
       System.out.println ("- Conjunto de prueba: "+dat.testInstances()+" patrones");
       System.out.println ();

       set  = dat.patterns();
       dist = new EuclideanDistance(set);

       System.out.println ("Agrupamiento basado en la matriz de similaridad\n");

       clustering = new SimilarityMatrix (cfg,set,dist,theta);

       clustering.printReport();
    }
  }
}