package rf.data;

// Data
// ----
// Format = ASCII file (vg: C4.5, LVQ...)

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import java.util.StringTokenizer;

/**
 * <p>
 * Input data modelling
 *
 * <p>
 * Format: Set of ASCII files (vg: C4.5, LVQ...)
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.AsciiDataset
 */


public class AsciiData implements Data
{
  // Variables de instancia
  // ----------------------

  int          atributos;  // Nmero de atributos

  AsciiDataset dataset;    // Casos de entrenamiento (todos)
  AsciiDataset learning;   // Casos de aprendizaje
  AsciiDataset test;       // Casos de prueba
  AsciiDataset unlabeled;  // Casos sin etiquetar (por clasificar)


  // Constructor
  // -----------

  public AsciiData (int attributes)
  {
    atributos = attributes;
    dataset   = new AsciiDataset(this);
    learning  = new AsciiDataset(this);
    test      = new AsciiDataset(this);
    unlabeled = new AsciiDataset(this);
  }

  // DATOS DE ENTRENAMIENTO (dataset = learning + test)

  public void setTraining (String fichero)
  {
    // Por defecto: 70% de las muestras para el aprendizaje
    //              30% de las muestras para la evaluacin

    setTraining (fichero,0.7);
  }

  public void setTraining (String fichero, double p)
  {
    int i;

    dataset = load (this, fichero);

    // Distribucin aleatoria de las muestras

    for (i=0; i<dataset.instances(); i++) {
        if (Math.random()<p) {
           learning.add ( dataset.pattern(i) );
        } else {
           test.add ( dataset.pattern(i) );
        }
    }
  }

  // DATOS DE APRENDIZAJE (learning)

  public void setLearning (String fichero)
  {
    int i;

    learning = load (this, fichero);

    for (i=0; i<learning.instances(); i++)
        dataset.add ( learning.pattern(i) );
  }

  // DATOS DE PRUEBA (test)

  public void setTest (String fichero)
  {
    int i;

    test = load (this, fichero);

    for (i=0; i<test.instances(); i++)
        dataset.add ( test.pattern(i) );
  }

  // DATOS POR CLASIFICAR (unlabeled)

  public void setUnlabeled (String fichero)
  {
    unlabeled = load (this, fichero);
  }

  // Lectura desde un fichero
  // ------------------------

  public static AsciiDataset load (Data data, String idFichero)
  {
    AsciiDataset     dataset = new AsciiDataset (data);
    Pattern          caso    = null;
    String           ascii   = null;
    RandomAccessFile fichero = null;

    try {

        fichero = new RandomAccessFile (idFichero, "r" );

        do {
            ascii = fichero.readLine();

            if (ascii!=null) {

               caso = parse (data, ascii.trim());

               if (caso!=null)
                  dataset.add (caso);
            }

        } while (ascii!=null);

        fichero.close();

    } catch (IOException e) {

        System.out.println(e.getMessage());
    }

    return dataset;
  }

  // Anlisis de una lnea del fichero ASCII

  private static Pattern parse (Data data, String ascii)
  {
    float[]         attr  = new float[data.attributes()];
    String          clase = null;
    String          token = null;
    StringTokenizer st    = new StringTokenizer(ascii, ", \t\r\n");;
    int             pos   = 0;

    token = nextToken(st);

    while (pos<=data.attributes() && token!=null) {

          if (token.equals("?")) {
             // Valor nulo
             return null;
          } else {

             if (pos==data.attributes()) {
                clase = token;
             } else {
                attr[pos] = new Float(token).floatValue();
             }

             token = nextToken(st);
             pos++;
          }
    }

    if (clase!=null)
       return new Pattern (attr,clase);
    else
       return null;
  }

  // nextToken

  private static String nextToken (StringTokenizer st)
  {
    if (st.hasMoreTokens())
       return st.nextToken().trim();
    else
       return null;
  }

  // Escritura en un fichero
  // -----------------------

  public static void save (Dataset set, String fileID)
  {
    int              i,j;
    String           line;
    File             filename;
    RandomAccessFile file;

    try {

        filename = new File(fileID);

        filename.delete();

        file     = new RandomAccessFile (filename, "rw" );

        for (i=0; i<set.instances(); i++) {
            line = set.pattern(i).toString() + "\n";
            file.write ( line.getBytes() );
        }

        file.close();

    } catch (IOException e) {

        System.out.println("ERROR: " + e.getMessage());
    }
  }


  // CONTABILIDAD GENERAL
  // --------------------

  // Nmero de atributos (sin contar la clase)

  public int attributes ()
  {
    return atributos;
  }

  // Nmero total de casos disponibles

  public int instances ()
  {
    return dataset.instances();
  }


  // CONJUNTO DE CASOS (PATRONES)
  // ----------------------------

  public Dataset patterns ()
  {
    return dataset;
  }

  // CONJUNTO DE APRENDIZAJE

  public Dataset learningSet ()
  {
    return learning;
  }

  // Nmero de casos del conjunto de aprendizaje

  public int learningInstances ()
  {
    return learning.instances();
  }

  // CONJUNTO DE PRUEBA

  public Dataset testSet ()
  {
    return test;
  }

  // Nmero de casos del conjunto de prueba

  public int testInstances ()
  {
    return test.instances();
  }

  // CONJUNTO DE DATOS POR CLASIFICAR

  public Dataset unlabeledSet ()
  {
    return unlabeled;
  }

  // Nmero de datos por clasificar

  public int unlabeledInstances ()
  {
    return unlabeled.instances();
  }

  // PRUEBA
  // ------

  public static void main (String args[])
  {
/**
    String       path      = "C:/etsii/rf/data/c45/";
    // BUPA
    int          atributos = 6; // 6+1
    String       datafile  = path + "bupa.dat";
/**
    // WINE
    int          atributos = 13; // 13+1
    String       datafile  = path + "wine.dat";
/**
    // IONO (clase no numrica)
    int          atributos = 34; // 34+1
    String       datafile  = path + "chars/iono.dat";
/**
    // LUNG-CAN (incluye valores nulos)
    int          atributos = 56; // 56+1
    String       datafile  = path + "nulls/lung-can.dat";
/**
    // BREAST-C (valores nulos)
    int          atributos = 10; // 10+1
    String       datafile  = path + "nulls/breast-c.dat";
/**/

    String datafile  = args[0];
    int    atributos = (new Integer(args[1])).intValue();

    // Conjunto de datos

    AsciiData data = new AsciiData (atributos);

    data.setTraining (datafile);

    // Info

    System.out.println(" Casos = "+data.instances());
    System.out.println(" - Aprendizaje "+data.learningInstances());
    System.out.println(" - Prueba "+data.testInstances());

    // Primer dato

    // System.out.println(data.patterns().pattern(0));

    data.save ( data.patterns(), "datos.out");
  }

}