package rf.data;

// Banda de una imagen
// -------------------
// Modelo de representacin: byte[]

// *******************
// Image band = byte[]
// *******************

// v1.0  Nov'98 Spanish identifiers
// v1.1  Dec'98 English identifiers

import java.awt.*;
import java.awt.image.MemoryImageSource;
import java.io.*;

/**
 * <p>
 * Input data modelling
 *
 * <p>
 * Format: Image file (8bpp)
 *
 * @version 1.1
 * @author  Fernando Berzal Galiano
 * @see     rf.data.ImageData
 * @see     rf.data.ImageDataset
 */

public class ImageBand
{
  // Variables de instancia
  // ----------------------

  byte datos[] = null;
  int  anchura = 0;
  int  altura  = 0;

  // Constructor
  // -----------

  public ImageBand (String fileID, int width, int height)
  {
    anchura = width;
    altura  = height;

    try {

        datos = byteArrayFromFile (fileID, width, height);

    } catch (IOException error) {

        // Error

        System.out.println ("ERROR: BAND "+fileID);
    }
  }

  // ------------------------------------------------
  // Lectura de un fichero con una imagen de tipo RAW
  // ------------------------------------------------

  private byte[] byteArrayFromStream (InputStream s)
                 throws IOException
  {
    int    n;
    int    count;
    int    length = s.available();
    byte[] buffer = new byte[length];

    count = 0;

    while (count<length) {

          n = s.read (buffer, count, length-count);

          if (n < 0)
             return null; // Error

          count += n;
    }

    return buffer;
  }

  private byte[] byteArrayFromFile (String fileID, int width, int height)
                 throws IOException
  {
    int              bytes    = -1;
    byte             buffer[] = new byte[width*height];
    RandomAccessFile f;


    f = new RandomAccessFile ( fileID, "r" );

    bytes = f.read (buffer,0,width*height);

    f.close();

    if (bytes==width*height)
       return buffer;
    else
       return null;
  }

  // -------------------------------------------
  // Obtencin de la imagen asociada a una banda
  // -------------------------------------------

  // NOTA: Los bytes en Java se consideran codificados en complemento a 2

  public int raw (byte b)
  {
    if (b<0)
       return 256+b;
    else
       return b;
  }

  // Conversor: byte[] -> Image

  public Image getRawImage ( )
  {
    return getRawImage (true, true, true);
  }

  // Conversor: byte[] -> Image (RGB)

  public Image getRawImage ( boolean r,         // Usar banda R
                             boolean g,         // Usar banda G
                             boolean b)         // Usar banda B
  {
    return getRawImage (r, g, b, 255);
  }

  // Conversor: byte[] -> Image (ARGB)

  public Image getRawImage ( boolean r,         // Usar banda R
                             boolean g,         // Usar banda G
                             boolean b,         // Usar banda B
                             int     alpha)     // Transparencia
  {
    int  i;
    int  pixels[] = new int[datos.length];
    int  color[]  = new int[256];
    int  val,max,min,rango;

    // RANGO

    max = min = raw(datos[0]);

    for (i=1; i<datos.length; i++) {

        val = raw(datos[i]);

        if (val>max)
           max = val;
        else if (val<min)
           min = val;
    }

    // System.out.println("MIN="+min+" MAX="+max);

    if (max>min) {

       rango = max-min+1;

       // PALETA

       for (i=0; i<256; i++) {

           // Nivel de gris
           // -------------

           // Tal cual
           // val = i;

           // Negativo
           // val = 255-i;

           // Transformacin lineal
              val = (int) (((i-min)*255.0)/rango);

           // Transformacin lineal negativada
           // val = (int) (((max-i)*255.0)/rango);

           // Transformacin logartmica
           // val = (int) (255.0 * Math.log ( (Math.exp(1)-1)*i/255.0 + 1 ));

           // Transformacin exponencial
           // val = (int) (255.0/(Math.exp(1)-1) * ( Math.exp(i/255.0) - 1 ));

           // Pixels: ALPHA+RED+GREEN+BLUE -> 0xAARRGGBB
           // ------------------------------------------

           color[i] = alpha<<24;

           if (r)
              color[i] |= (val<<16);

           if (g)
              color[i] |= (val<<8);

           if (b)
              color[i] |= val;
       }

       // IMAGEN

       for (i=0; i<datos.length; i++)
           pixels[i] = color[raw(datos[i])];
    }

    // RESULTADO

    return Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(anchura, altura, pixels, 0, anchura));
  }

  // ------------------------------------------
  // Obtencin de todos los valores de la banda
  // ------------------------------------------

  public float[] floatArray ()
  {
    int   i;
    float array[] = new float[datos.length];

    for (i=0; i<datos.length; i++)
        array[i] = (float) raw(datos[i]);

    return array;
  }

  // ------------------------------------------
  // Obtencin de un valor concreto de la banda
  // ------------------------------------------

  public float value (int k)
  {
    if (k>=0 && k<datos.length)
       return (float) raw(datos[k]);
    else
       return 0;
  }

  public float value (int x, int y)
  {
    if (x>=0 && x<anchura && y>=0 && y<altura)
       return (float) raw(datos[x*anchura+y]);
    else
       return 0;
  }

}