package rf.gui;

// Title:       Numerical Cruncher
// Version:     1.2
// Copyright:   1999
// Author:      Fernando Berzal Galiano
// E-mail:      berzal@acm.org

import java.awt.*;
import java.awt.event.*;

import fbg.gui.*;
import fbg.gui.monitor.*;

import fbg.util.*;

import rf.*;
import rf.data.*;
import rf.edit.*;
import rf.classification.*;
import rf.classification.parametric.*;
import rf.classification.nn.*;
import rf.classification.lvq.*;
import rf.clustering.*;
import rf.distance.*;

import tminer.TMiner;

// v1.0 Jan'99 JQuest
// v1.1 May'99 Classifiers: Cancel button
// v1.2 Jun'99 GUI (icon + memory monitor)
// v2.0 Jun'99 TMiner Bridge

public class MainMenu extends Frame
{
  // Data

  DataConfiguration cfg = null;
  Dataset           set = null;
  Dataset           all = null;

  // GUI logo

  Logo      logo = new Logo();

  // Output messages

  Output    out  = null;

  // Help system

  Help      help = null;

  // TMiner bridge

  public TMiner miner = null;

  // Application or applet ?

  public boolean standalone = false;

  // Menu options

  private static final byte STATS          = 1;
  private static final byte CHART2D        = 2;
  private static final byte CLASSES        = 3;

  private static final byte EDIT_UNDO      = 10;
  private static final byte EDIT_WILSON    = 11;
  private static final byte EDIT_PARTITION = 12;
  private static final byte EDIT_MULTIEDIT = 13;
  private static final byte EDIT_HART      = 14;

  private static final byte CLASS_EUCLIDEAN      = 51;
  private static final byte CLASS_NORMALIZED     = 52;
  private static final byte CLASS_MAHALANOBIS    = 53;
  private static final byte CLASS_QUADRATIC      = 54;
  private static final byte CLASS_PARALLELEPIPED = 55;
  private static final byte CLASS_1NN            = 56;
  private static final byte CLASS_KNN            = 57;
  private static final byte CLASS_LVQ1           = 58;
  private static final byte CLASS_OLVQ1          = 59;
  private static final byte CLASS_DSM            = 60;

  private static final byte CLUSTER_ADAPTIVE   = 71;
  private static final byte CLUSTER_BW         = 72;
  private static final byte CLUSTER_KMEANS     = 73;
  private static final byte CLUSTER_GRASP      = 74;
  private static final byte CLUSTER_SEQUENTIAL = 75;
  private static final byte CLUSTER_ISODATA    = 76;
  private static final byte CLUSTER_SIMILARITY = 77;

  private static final byte HELP   = 100;
  private static final byte MEMORY = 101;
  private static final byte ABOUT  = 102;

  // Menu

  MenuBar        menu = new MenuBar();

  Menu           menuDatos         = new Menu();
  MenuItem       menuItemStats     = new MenuItem();
  MenuItem       menuItem2D        = new MenuItem();
  MenuItem       menuItemExit      = new MenuItem();

  Menu           menuEdit          = new Menu();
  MenuItem       menuItemUndo      = new MenuItem();
  MenuItem       menuItemWilson    = new MenuItem();
  MenuItem       menuItemPartition = new MenuItem();
  MenuItem       menuItemMultiEdit = new MenuItem();
  MenuItem       menuItemHart      = new MenuItem();

  Menu           menuClassification   = new Menu();
  MenuItem       menuItemClasses      = new MenuItem();
  Menu           menuParametric       = new Menu();
  MenuItem       menuItemEuclidean    = new MenuItem();
  MenuItem       menuItemNormalized   = new MenuItem();
  MenuItem       menuItemMahalanobis  = new MenuItem();
  MenuItem       menuItemQuadratic    = new MenuItem();
  MenuItem       menuItemParallelepid = new MenuItem();
  Menu           menuNN               = new Menu();
  MenuItem       menuItem1NN          = new MenuItem();
  MenuItem       menuItemKNN          = new MenuItem();
  Menu           menuLVQ              = new Menu();
  MenuItem       menuItemLVQ1         = new MenuItem();
  MenuItem       menuItemOLVQ1        = new MenuItem();
  MenuItem       menuItemDSM          = new MenuItem();

  Menu           menuClustering       = new Menu();
  MenuItem       menuItemAdaptive     = new MenuItem();
  MenuItem       menuItemBW           = new MenuItem();
  MenuItem       menuItemKMEANS       = new MenuItem();
  MenuItem       menuItemGRASP        = new MenuItem();
  MenuItem       menuItemSequential   = new MenuItem();
  MenuItem       menuItemISODATA      = new MenuItem();
  MenuItem       menuItemSimilarity   = new MenuItem();

  Menu           menuHelp = new Menu();
  MenuItem       menuItemAbout = new MenuItem();
  MenuItem       menuItemMemory = new MenuItem();
  MenuItem       menuItemHelp = new MenuItem();


  // Constructor
  // -----------

  public MainMenu (DataConfiguration cfg, Output output, Help help_system, boolean standalone)
  {
    this (cfg,output,help_system,null);

    this.standalone = standalone;
  }

  public MainMenu (DataConfiguration data_cfg, Output output, Help help_system, TMiner tminer)
  {
    cfg   = data_cfg;
    set   = cfg.getData().learningSet();
    all   = cfg.getData().patterns();
    out   = output;
    help  = help_system;
    miner = tminer; 

    if (miner!=null)
       standalone = miner.standalone;

    // ---
    // GUI
    // ---

    try {

      init();
      this.setTitle(cfg.getID());
      this.setSize(400,300);
      this.add (logo);

      if (cfg.totalClasses()>0) {
         menuClassification.setEnabled(true);
         menuEdit.setEnabled(true);
      } else {
         menuClassification.setEnabled(false);
         menuEdit.setEnabled(false);
      }

      menuItemUndo.setEnabled(false);

    } catch (Exception e) {
      e.printStackTrace();
    }

    if (Environment.settings==Environment.SPANISH)
       Spanish();
    else
       English();
  }


  private void init() throws Exception
  {
    Image icon = Loader.loadImage(getClass(),"image/icon.gif");

    this.setIconImage(icon);
    this.setBackground(Color.lightGray);
    this.setMenuBar(menu);
    this.addWindowListener(new MainMenu_windowAdapter(this));

    menu.setFont(new Font("Dialog", 0, 12));

    // Event handling

    menuItemStats.addActionListener(new MainMenu_menuItem_actionAdapter(this,STATS));
    menuItem2D.addActionListener(new MainMenu_menuItem_actionAdapter(this,CHART2D));
    menuItemClasses.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASSES));

    menuItemExit.addActionListener(new MainMenu_menuItemExit_actionAdapter(this));

    menuItemAbout.addActionListener(new MainMenu_menuItem_actionAdapter(this,ABOUT));
    menuItemMemory.addActionListener(new MainMenu_menuItem_actionAdapter(this,MEMORY));
    menuItemHelp.addActionListener(new MainMenu_menuItem_actionAdapter(this,HELP));

    menuItemUndo.addActionListener(new MainMenu_menuItem_actionAdapter(this,EDIT_UNDO));
    menuItemWilson.addActionListener(new MainMenu_menuItem_actionAdapter(this,EDIT_WILSON));
    menuItemPartition.addActionListener(new MainMenu_menuItem_actionAdapter(this,EDIT_PARTITION));
    menuItemMultiEdit.addActionListener(new MainMenu_menuItem_actionAdapter(this,EDIT_MULTIEDIT));
    menuItemHart.addActionListener(new MainMenu_menuItem_actionAdapter(this,EDIT_HART));

    menuItemEuclidean.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_EUCLIDEAN));
    menuItemNormalized.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_NORMALIZED));
    menuItemMahalanobis.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_MAHALANOBIS));
    menuItemQuadratic.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_QUADRATIC));
    menuItemParallelepid.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_PARALLELEPIPED));
    menuItem1NN.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_1NN));
    menuItemKNN.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_KNN));
    menuItemLVQ1.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_LVQ1));
    menuItemOLVQ1.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_OLVQ1));
    menuItemDSM.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLASS_DSM));

    menuItemAdaptive.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_ADAPTIVE));
    menuItemBW.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_BW));
    menuItemKMEANS.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_KMEANS));
    menuItemGRASP.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_GRASP));
    menuItemSequential.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_SEQUENTIAL));
    menuItemISODATA.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_ISODATA));
    menuItemSimilarity.addActionListener(new MainMenu_menuItem_actionAdapter(this,CLUSTER_SIMILARITY));

    // Menu

    this.setMenuBar(menu);
    menu.add(menuDatos);
    menu.add(menuEdit);
    menu.add(menuClassification);
    menu.add(menuClustering);
    menu.add(menuHelp);

    menuDatos.add(menuItemStats);
    menuDatos.add(menuItem2D);
    menuDatos.addSeparator();
    menuDatos.add(menuItemExit);

    menuEdit.add(menuItemUndo);
    menuEdit.addSeparator();
    menuEdit.add(menuItemWilson);
    menuEdit.add(menuItemPartition);
    menuEdit.add(menuItemMultiEdit);
    menuEdit.addSeparator();
    menuEdit.add(menuItemHart);

    menuClassification.add(menuItemClasses);
    menuClassification.addSeparator();
    menuClassification.add(menuParametric);
    menuClassification.add(menuNN);
    menuClassification.add(menuLVQ);

    menuParametric.add(menuItemEuclidean);
    menuParametric.add(menuItemNormalized);
    menuParametric.add(menuItemMahalanobis);
    menuParametric.add(menuItemQuadratic);
    menuParametric.add(menuItemParallelepid);

    menuNN.add(menuItem1NN);
    menuNN.add(menuItemKNN);

    menuLVQ.add(menuItemLVQ1);
    menuLVQ.add(menuItemOLVQ1);
    menuLVQ.add(menuItemDSM);

    menuClustering.add(menuItemAdaptive);
    menuClustering.add(menuItemBW);
    menuClustering.add(menuItemKMEANS);
    menuClustering.add(menuItemGRASP);
    menuClustering.add(menuItemSequential);
    menuClustering.add(menuItemISODATA);
    menuClustering.add(menuItemSimilarity);

    menuHelp.add(menuItemHelp);
    menuHelp.addSeparator();
    menuHelp.add(menuItemMemory);
    menuHelp.addSeparator();
    menuHelp.add(menuItemAbout);
  }

  // Spanish version

  private void Spanish ()
  {
    menuDatos.setLabel("Datos");
    menuEdit.setLabel("Edicin");
    menuClassification.setLabel("Clasificacin");
    menuClustering.setLabel("Agrupamiento");
    menuHelp.setLabel("Ayuda");

    menuItemStats.setLabel("Estadsticas...");
    menuItem2D.setLabel("Representacin 2D...");

    menuItemExit.setLabel ("Salir");

    menuItemUndo.setLabel ("Deshacer edicin");
    menuItemWilson.setLabel ("Edicin de Wilson");
    menuItemPartition.setLabel ("Edicin por particiones");
    menuItemMultiEdit.setLabel ("Multiedicin");
    menuItemHart.setLabel ("Condensado de Hart");

    menuItemClasses.setLabel ("Estadsticas por clases...");
    menuParametric.setLabel("Modelos paramtricos");
    menuItemEuclidean.setLabel("Distancia eucldea");
    menuItemNormalized.setLabel("Distancia eucldea normalizada");
    menuItemMahalanobis.setLabel("Distancia de Mahalanobis");
    menuItemQuadratic.setLabel("Clasificador cuadrtico");
    menuItemParallelepid.setLabel("Mtodo de los paraleleppedos");
    menuNN.setLabel("Vecino ms cercano (NN)");
    menuItem1NN.setLabel("1-NN");
    menuItemKNN.setLabel("k-NN");
    menuLVQ.setLabel("Aprendizaje adaptativo");
    menuItemLVQ1.setLabel("LVQ-1");
    menuItemOLVQ1.setLabel("OLVQ-1");
    menuItemDSM.setLabel("DSM");

    menuItemAdaptive.setLabel("Algoritmo adaptativo");
    menuItemBW.setLabel("Algoritmo de Batchelor & Wilkins");
    menuItemKMEANS.setLabel("Algoritmo de las K medias");
    menuItemGRASP.setLabel("Algoritmo GRASP");
    menuItemSequential.setLabel("Algoritmo secuencial");
    menuItemISODATA.setLabel("Algoritmo ISODATA");
    menuItemSimilarity.setLabel("Matriz de similaridad");

    menuItemAbout.setLabel("Acerca de...");
    menuItemMemory.setLabel("Uso de memoria...");
    menuItemHelp.setLabel ("Documentacin");
  }

  // EnglishVersion

  private void English ()
  {
    menuDatos.setLabel("Data");
    menuEdit.setLabel("Edit");
    menuClassification.setLabel("Classification");
    menuClustering.setLabel("Clustering");
    menuHelp.setLabel("Help");

    menuItemStats.setLabel("Statistics...");
    menuItem2D.setLabel("2D Plot...");

    menuItemExit.setLabel ("Exit");

    menuItemUndo.setLabel ("Undo edition");
    menuItemWilson.setLabel ("Wilson's edition");
    menuItemPartition.setLabel ("Partition edition");
    menuItemMultiEdit.setLabel ("Multiedit");
    menuItemHart.setLabel ("Hart's condensation");

    menuItemClasses.setLabel ("Class Statistics...");
    menuParametric.setLabel("Parametric Models");
    menuItemEuclidean.setLabel("Euclidean Distance");
    menuItemNormalized.setLabel("Normalized Euclidean Distance");
    menuItemMahalanobis.setLabel("Mahalanobis' Distance");
    menuItemQuadratic.setLabel("Quadratic Classifier");
    menuItemParallelepid.setLabel("Parallelepipeds Method");
    menuNN.setLabel("Nearest Neighbour(s)");
    menuItem1NN.setLabel("1-NN");
    menuItemKNN.setLabel("k-NN");
    menuLVQ.setLabel("Adaptive Learning");
    menuItemLVQ1.setLabel("LVQ-1");
    menuItemOLVQ1.setLabel("OLVQ-1");
    menuItemDSM.setLabel("DSM");

    menuItemAdaptive.setLabel("Adaptive algorithm");
    menuItemBW.setLabel("Batchelor & Wilkins");
    menuItemKMEANS.setLabel("K-Means");
    menuItemGRASP.setLabel("GRASP");
    menuItemSequential.setLabel("Sequential algorithm");
    menuItemISODATA.setLabel("ISODATA");
    menuItemSimilarity.setLabel("Similarity Matrix");

    menuItemAbout.setLabel("About...");
    menuItemMemory.setLabel("Memory...");
    menuItemHelp.setLabel ("On-line help");
  }

  // Exit
  // ----

  void exit ()
  {
    dispose();
  }


  // Menu options
  // ------------

  void actionPerformed (byte option)
  {
    Frame2D      frame2D;
    FrameStats   frameStats;
    FrameClasses frameClasses;
    Dimension    d = Toolkit.getDefaultToolkit().getScreenSize();
    Benchmark    crono = new Benchmark();

    GUI.setWaitCursor(this);

    switch (option) {

      case STATS:

           if (Environment.settings==Environment.SPANISH)
              out.message("Obteniendo estadsticas... ");
           else
              out.message("Obtaining data stats...");

           crono.start();
           frameStats = new FrameStats (cfg, cfg.getData().patterns() );
           frameStats.setLocation((d.width - frameStats.getSize().width) / 2, (d.height - frameStats.getSize().height) / 2);
           frameStats.setVisible(true);
           crono.stop();
           out.message(crono+"\n");
           break;

      case CHART2D:
           frame2D = new Frame2D (cfg, cfg.getData().patterns() );
           frame2D.setLocation((d.width - frame2D.getSize().width) / 2, (d.height - frame2D.getSize().height) / 2);
           frame2D.setVisible(true);
           break;

      case CLASSES:

           if (Environment.settings==Environment.SPANISH)
              out.message("Obteniendo estadsticas por clases... ");
           else
              out.message("Obtaining data stats for the different classes...");

           crono.start();
           frameClasses = new FrameClasses (cfg, cfg.getData().patterns() );
           frameClasses.setLocation((d.width - frameClasses.getSize().width) / 2, (d.height - frameClasses.getSize().height) / 2);
           frameClasses.setVisible(true);
           crono.stop();
           out.message(crono+"\n");
           break;

      case EDIT_UNDO:
      case EDIT_WILSON:
      case EDIT_PARTITION:
      case EDIT_MULTIEDIT:
      case EDIT_HART:
           edit (option);
           break;

      case CLASS_EUCLIDEAN:
      case CLASS_NORMALIZED:
      case CLASS_MAHALANOBIS:
      case CLASS_QUADRATIC:
      case CLASS_PARALLELEPIPED:
      case CLASS_1NN:
      case CLASS_KNN:
      case CLASS_LVQ1:
      case CLASS_OLVQ1:
      case CLASS_DSM:
           buildClassifier (option);
           break;

      case CLUSTER_ADAPTIVE:
      case CLUSTER_BW:
      case CLUSTER_KMEANS:
      case CLUSTER_GRASP:
      case CLUSTER_SEQUENTIAL:
      case CLUSTER_ISODATA:
      case CLUSTER_SIMILARITY:
           buildClusters (option);
           break;

      case ABOUT:
           aboutNC();
	   break;

      case MEMORY:
           memory();
	   break;

      case HELP:
           onlineHelp();
	   break;
    }

    GUI.setDefaultCursor (this);
  }

  private void buildClassifier (byte option)
  {
    DialogApriori   dialogApriori = null;
    DialogK         dialogK       = null;
    Dialog2INT      dialog2       = null;
    FrameClassifier frameClassifier = null;
    Dimension       dim = Toolkit.getDefaultToolkit().getScreenSize();
    boolean         ok            = true;
    boolean         useApriori    = false;
    int             k             = 2;
    int             np            = 7*cfg.totalClasses();
    int             steps         = set.instances();
    float           learningRate  = 0.1f;
    Classifier      classifier;
    String          label,label1,label2;
    Benchmark       crono;

    // Parmetros

    switch (option) {

      case CLASS_EUCLIDEAN:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador lineal: Distancia eucldea\n");
              label = "Clasificador lineal";
           } else {
              out.message ("Linear classifier: Euclidean distance\n");
              label = "Linear classifier";
           }

           dialogApriori = new DialogApriori(this,label);
           dialogApriori.setLocation((dim.width - dialogApriori.getSize().width) / 2, (dim.height - dialogApriori.getSize().height) / 2);
           dialogApriori.show();

           ok         = dialogApriori.ok;
           useApriori = dialogApriori.useAprioriProbabilities();
           break;

      case CLASS_NORMALIZED:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador lineal: Distancia eucldea normalizada\n");
              label = "Clasificador lineal";
           } else {
              out.message ("Linear classifier: Normalized Euclidean distance\n");
              label = "Linear classifier";
           }

           dialogApriori = new DialogApriori(this,label);
           dialogApriori.setLocation((dim.width - dialogApriori.getSize().width) / 2, (dim.height - dialogApriori.getSize().height) / 2);
           dialogApriori.show();

           ok         = dialogApriori.ok;
           useApriori = dialogApriori.useAprioriProbabilities();
           break;

      case CLASS_MAHALANOBIS:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador lineal: Distancia de Mahalanobis\n");
              label = "Clasificador lineal";
           } else {
              out.message ("Linear classifier: Mahalanobis' distance\n");
              label = "Linear classifier";
           }

           dialogApriori = new DialogApriori(this,label);
           dialogApriori.setLocation((dim.width - dialogApriori.getSize().width) / 2, (dim.height - dialogApriori.getSize().height) / 2);
           dialogApriori.show();

           ok         = dialogApriori.ok;
           useApriori = dialogApriori.useAprioriProbabilities();
           break;

      case CLASS_QUADRATIC:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador cuadrtico\n");
           } else {
              out.message ("Quadratic classifier\n");
           }
           break;

      case CLASS_PARALLELEPIPED:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Mtodo de los paraleleppedos\n");
           } else {
              out.message ("Parallelepipeds Classification Method\n");
           }
           break;

      case CLASS_1NN:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador 1-NN\n");
           } else {
              out.message ("1-NN Classifier\n");
           }
           break;

      case CLASS_KNN:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador k-NN\n");
              label = "Clasificador k-NN";
           } else {
              out.message ("k-NN Classifier\n");
              label = "k-NN Classifier";
           }

           dialogK = new DialogK (this, label);
           dialogK.setLocation((dim.width - dialogK.getSize().width) / 2, (dim.height - dialogK.getSize().height) / 2);
           dialogK.show();

           ok = dialogK.ok;
           k  = dialogK.K();
           break;

      case CLASS_LVQ1:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador LVQ-1\n");
           } else {
              out.message ("LVQ-1 Classifier\n");
           }

           dialogApriori = new DialogApriori(this,"LVQ1");
           dialogApriori.setLocation((dim.width - dialogApriori.getSize().width) / 2, (dim.height - dialogApriori.getSize().height) / 2);
           dialogApriori.show();

           ok         = dialogApriori.ok;
           useApriori = dialogApriori.useAprioriProbabilities();

           if (ok) {

              if (Environment.settings==Environment.SPANISH) {

                 if (useApriori)
                    label1 = "Nmero de prototipos";
                 else
                    label1 = "Prototipos por clase";

                 label2 = "Nmero de iteraciones";

              } else {

                 if (useApriori)
                    label1 = "Number of codebooks";
                 else
                    label1 = "Codebooks per class";

                 label2 = "Data scans";
              }

              dialog2 = new Dialog2INT (this, "LVQ-1", label1, label2);
              dialog2.setLocation((dim.width - dialog2.getSize().width) / 2, (dim.height - dialog2.getSize().height) / 2);
              dialog2.show();

              ok = dialog2.ok;

              if (useApriori)
                 np = dialog2.int1();
              else
                 np = dialog2.int1()*cfg.totalClasses();

              steps = dialog2.int2()*set.instances();

              learningRate = 0.03f;
           }
           break;

      case CLASS_OLVQ1:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador OLVQ-1\n");
           } else {
              out.message ("OLVQ-1 Classifier\n");
           }

           dialogApriori = new DialogApriori(this,"OLVQ-1");
           dialogApriori.setLocation((dim.width - dialogApriori.getSize().width) / 2, (dim.height - dialogApriori.getSize().height) / 2);
           dialogApriori.show();

           ok         = dialogApriori.ok;
           useApriori = dialogApriori.useAprioriProbabilities();

           if (ok) {

              if (Environment.settings==Environment.SPANISH) {

                 if (useApriori)
                    label1 = "Nmero de prototipos";
                 else
                    label1 = "Prototipos por clase";

                 label2 = "Nmero de iteraciones";

              } else {

                 if (useApriori)
                    label1 = "Number of codebooks";
                 else
                    label1 = "Codebooks per class";

                 label2 = "Data scans";
              }

              dialog2 = new Dialog2INT (this, "OLVQ-1", label1, label2);
              dialog2.setLocation((dim.width - dialog2.getSize().width) / 2, (dim.height - dialog2.getSize().height) / 2);
              dialog2.show();

              ok = dialog2.ok;

              if (useApriori)
                 np = dialog2.int1();
              else
                 np = dialog2.int1()*cfg.totalClasses();

              steps = dialog2.int2()*set.instances();

              learningRate = 0.3f;
           }

           break;

      case CLASS_DSM:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Clasificador DSM\n");
           } else {
              out.message ("DSM Classifier\n");
           }

           dialogApriori = new DialogApriori(this,"DSM");
           dialogApriori.setLocation((dim.width - dialogApriori.getSize().width) / 2, (dim.height - dialogApriori.getSize().height) / 2);
           dialogApriori.show();

           ok         = dialogApriori.ok;
           useApriori = dialogApriori.useAprioriProbabilities();

           if (ok) {
           
              if (Environment.settings==Environment.SPANISH) {

                 if (useApriori)
                    label1 = "Nmero de prototipos";
                 else
                    label1 = "Prototipos por clase";

                 label2 = "Nmero de iteraciones";

              } else {

                 if (useApriori)
                    label1 = "Number of codebooks";
                 else
                    label1 = "Codebooks per class";

                 label2 = "Data scans";
              }

              dialog2 = new Dialog2INT (this, "DSM", label1, label2);
              dialog2.setLocation((dim.width - dialog2.getSize().width) / 2, (dim.height - dialog2.getSize().height) / 2);
              dialog2.show();

              ok = dialog2.ok;

              if (useApriori)
                 np = dialog2.int1();
              else
                 np = dialog2.int1()*cfg.totalClasses();

              steps = dialog2.int2()*set.instances();

              learningRate = 0.03f;
           }
           break;
    }

    // Construccin del clasificador

    classifier = null;

    if (ok) {

       if (Environment.settings==Environment.SPANISH)
          out.message("Construccin del clasificador... ");
       else
          out.message("Building classifier... ");

       crono = new Benchmark();
       crono.start();

       switch (option) {

         case CLASS_EUCLIDEAN:
              classifier = new EuclideanClassifier(cfg,set,useApriori);
              break;

         case CLASS_NORMALIZED:
              classifier = new NormalizedEuclideanClassifier(cfg,set,useApriori);
              break;

         case CLASS_MAHALANOBIS:
              try {
                 classifier = new MahalanobisClassifier(cfg,set,useApriori);
              } catch (ArithmeticException e) {

                 if (Environment.settings==Environment.SPANISH)
                    label = "Matriz de covarianza singular";
                 else
                    label = "Singular covariance matrix";

                 Message.showMessageDialog ( this, label, "Error", Message.ERROR_MESSAGE);
              }
              break;

         case CLASS_QUADRATIC:
              try {
                 classifier = new QuadraticClassifier(cfg,set);
              } catch (ArithmeticException e) {

                 if (Environment.settings==Environment.SPANISH)
                    label = "Matriz de covarianza singular";
                 else
                    label = "Singular covariance matrix";

                 Message.showMessageDialog ( this, label, "Error", Message.ERROR_MESSAGE);
              }
              break;

         case CLASS_PARALLELEPIPED:
              classifier = new ParallelepipedClassifier(cfg,set);
              break;

         case CLASS_1NN:
              classifier = new NNClassifier(cfg,set);
              break;

         case CLASS_KNN:
              classifier = new kNNClassifier(cfg,set,k);
              break;

         case CLASS_LVQ1:
              classifier = new lvq1 (cfg,set,np,steps,learningRate,useApriori);
              break;

         case CLASS_OLVQ1:
              classifier = new olvq1 (cfg,set,np,steps,learningRate,useApriori);
              break;

         case CLASS_DSM:
              classifier = new dsm (cfg,set,np,steps,learningRate,useApriori);
              break;
       }

       crono.stop();
       out.message(crono+"\n");

    }

    // Evaluacin del clasificador

    if (classifier!=null) {

       switch (option) {
         case CLASS_EUCLIDEAN:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador lineal: Distancia eucldea";
              else
                 label = "Linear classifier: Euclidean distance";
              break;

         case CLASS_NORMALIZED:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador lineal: Distancia eucldea normalizada";
              else
                 label = "Linear classifier: Normalized euclidean distance";
              break;

         case CLASS_MAHALANOBIS:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador lineal: Distancia de Mahalanobis";
              else
                 label = "Linear classifier: Mahalanobis' distance";
              break;

         case CLASS_QUADRATIC:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador cuadrtico";
              else
                 label = "Quadratic classifier";
              break;

         case CLASS_PARALLELEPIPED:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador paramtrico: Mtodo de los paraleleppedos";
              else
                 label = "Parametric classifier: Parallelepipeds Method";
              break;

         case CLASS_1NN:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador 1-NN";
              else
                 label = "Nearest neighbour classifier";
              break;

         case CLASS_KNN:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador "+k+"-NN";
              else
                 label = k+"-NN classifier";
              break;

         case CLASS_LVQ1:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador LVQ-1";
              else
                 label = "LVQ-1 classifier";
              break;

         case CLASS_OLVQ1:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador OLVQ-1";
              else
                 label = "OLVQ-1 classifier";
              break;

         case CLASS_DSM:
              if (Environment.settings==Environment.SPANISH)
                 label = "Clasificador DSM";
              else
                 label = "DSM classifier";
              break;

         default:
              label = null;
       }

       if (Environment.settings==Environment.SPANISH)
          out.message("Evaluacin del clasificador... ");
       else
          out.message("Classifier evaluation... ");

       crono = new Benchmark();
       crono.start();

       frameClassifier = new FrameClassifier (cfg, classifier, set, cfg.getData().testSet(), label, out, standalone);
       frameClassifier.setLocation((dim.width - frameClassifier.getSize().width) / 2, (dim.height - frameClassifier.getSize().height) / 2);
       frameClassifier.setVisible(true);

       crono.stop();
       out.message(crono+"\n");
    }
  }

  private void edit (byte option)
  {
    DialogK    dialogK = null;
    Dialog2INT dialog2 = null;
    Dimension  dim = Toolkit.getDefaultToolkit().getScreenSize();
    Benchmark  crono = new Benchmark();
    String     label;
    int        k=2,m=2,I=2;

    // Parmetros

    switch (option) {

      case EDIT_WILSON:

           if (Environment.settings==Environment.SPANISH)
              label = "Edicin de Wilson";
           else
              label = "Wilson's edition";

           dialogK = new DialogK (this, label);
           dialogK.setLocation((dim.width - dialogK.getSize().width) / 2, (dim.height - dialogK.getSize().height) / 2);
           dialogK.show();
           k = dialogK.K();
           break;

      case EDIT_PARTITION:

           if (Environment.settings==Environment.SPANISH)
              label = "Edicin por particiones";
           else
              label = "Partition edition";

           dialog2 = new Dialog2INT (this, label, "m", "k");
           dialog2.setLocation((dim.width - dialog2.getSize().width) / 2, (dim.height - dialog2.getSize().height) / 2);
           dialog2.show();
           m = dialog2.int1();
           k = dialog2.int2();
           break;

      case EDIT_MULTIEDIT:
           dialog2 = new Dialog2INT (this, "Multiedit", "m", "I");
           dialog2.setLocation((dim.width - dialog2.getSize().width) / 2, (dim.height - dialog2.getSize().height) / 2);
           dialog2.show();
           m = dialog2.int1();
           I = dialog2.int2();
           break;
    }

    // Edicin

    crono.start();

    switch (option) {

      case EDIT_UNDO:

           if (Environment.settings==Environment.SPANISH)
              out.message("Restablecimiento del conjunto de entrenamiento original\n");
           else
              out.message("Restoring original training dataset\n");

           set = cfg.getData().learningSet();
           menuItemUndo.setEnabled(false);
           break;

      case EDIT_WILSON:

           if (Environment.settings==Environment.SPANISH)
              out.message("Edicin de Wilson (K="+k+")\n");
           else
              out.message("Wilson's edition (K="+k+")\n");

           menuItemUndo.setEnabled(true);
           set = (new WilsonEdition(cfg,set,k)).getResult();
           break;

      case EDIT_PARTITION:

           if (Environment.settings==Environment.SPANISH)
              out.message("Edicin por particiones (m="+m+", k="+k+")\n");
           else
              out.message("Partition edition (m="+m+", k="+k+")\n");

           menuItemUndo.setEnabled(true);
           set = (new PartitionEdition(cfg,set,m,k)).getResult();
           break;

      case EDIT_MULTIEDIT:
           out.message("Multiedit  (m="+m+", I="+I+")\n");
           menuItemUndo.setEnabled(true);
           set = (new MultiEdit(cfg,set,m,I)).getResult();
           break;

      case EDIT_HART:

           if (Environment.settings==Environment.SPANISH)
              out.message("Condensado de Hart\n");
           else
              out.message("Hart's condensation algorithm\n");

           menuItemUndo.setEnabled(true);
           set = (new Hart(cfg,set)).getResult();
           break;
    }

    crono.stop();

    if (Environment.settings==Environment.SPANISH) {
       out.message("Tiempo empleado: "+crono+"\n");
       out.message("Tamao final del conjunto de entrenamiento: "+set.instances()+" patrones\n");
    } else {
       out.message("Consumed time: "+crono+"\n");
       out.message("Training set final size: "+set.instances()+" patterns\n");
    }
  }

  // Mtodos de agrupamiento (Clustering algorithms)
  // -----------------------------------------------
  // - Adaptive Clustering: CLUSTER_ADAPTIVE
  // - Batchelor & Wilkins' Algorithm: CLUSTER_BW
  // - K Means: CLUSTER_KMEANS
  // - GRASP Technique: CLUSTER_GRASP
  // - Sequential Clustering: CLUSTER_SEQUENTIAL
  // - ISODATA Algorithm: CLUSTER_ISODATA
  // - Similarity Matrix: CLUSTER_SIMILARITY

  private void buildClusters (byte option)
  {
    Dataset  patterns;
    Distance dm;

    DialogAdaptiveClustering   dialogAC      = null;
    DialogBatchelorWilkins     dialogBW      = null;
    DialogKMeans               dialogKMeans  = null;
    DialogGRASPClustering      dialogGRASP   = null;
    DialogSequentialClustering dialogSC      = null;
    DialogISODATA              dialogISODATA = null;
    DialogSimilarityMatrix     dialogSM      = null;

    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();


    // Data

    if (cfg.totalClasses()>0) {
       patterns = set;
    } else {
       patterns = all;
    }


    // Distance metrics

    dm  = new EuclideanDistance(patterns);


    // Parameters

    switch (option) {

      case CLUSTER_ADAPTIVE:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Mtodo de agrupamiento adaptativo\n");
           } else {
              out.message ("Adaptive Clustering\n");
           }

           dialogAC = new DialogAdaptiveClustering (cfg,patterns,dm,this,out,help);
           dialogAC.setLocation((dim.width - dialogAC.getSize().width) / 2, (dim.height - dialogAC.getSize().height) / 2);
           dialogAC.show();
           break;

      case CLUSTER_BW:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Algoritmo de Batchelor y Wilkins\n");
           } else {
              out.message ("Batchelor and Wilkins' Algorithm\n");
           }

           dialogBW = new DialogBatchelorWilkins (cfg,patterns,dm,this,out,help);
           dialogBW.setLocation((dim.width - dialogBW.getSize().width) / 2, (dim.height - dialogBW.getSize().height) / 2);
           dialogBW.show();
           break;

      case CLUSTER_KMEANS:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Algoritmo de las K Medias\n");
           } else {
              out.message ("K-Means\n");
           }

           dialogKMeans = new DialogKMeans (cfg,patterns,dm,this,out,help);
           dialogKMeans.setLocation((dim.width - dialogKMeans.getSize().width) / 2, (dim.height - dialogKMeans.getSize().height) / 2);
           dialogKMeans.show();
           break;

      case CLUSTER_GRASP:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Tcnica GRASP\n");
           } else {
              out.message ("GRASP Technique\n");
           }

           dialogGRASP = new DialogGRASPClustering (cfg,patterns,dm,this,out,help);
           dialogGRASP.setLocation((dim.width - dialogGRASP.getSize().width) / 2, (dim.height - dialogGRASP.getSize().height) / 2);
           dialogGRASP.show();
           break;

      case CLUSTER_SEQUENTIAL:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Algoritmo de agrupamiento secuencial\n");
           } else {
              out.message ("Sequential Clustering\n");
           }

           dialogSC = new DialogSequentialClustering (cfg,patterns,dm,this,out,help);
           dialogSC.setLocation((dim.width - dialogSC.getSize().width) / 2, (dim.height - dialogSC.getSize().height) / 2);
           dialogSC.show();
           break;

      case CLUSTER_ISODATA:
           out.message ("ISODATA\n");
           dialogISODATA = new DialogISODATA (cfg,patterns,dm,this,out,help);
           dialogISODATA.setLocation((dim.width - dialogISODATA.getSize().width) / 2, (dim.height - dialogISODATA.getSize().height) / 2);
           dialogISODATA.show();
           break;

      case CLUSTER_SIMILARITY:

           if (Environment.settings==Environment.SPANISH) {
              out.message ("Algoritmo basado en la matriz de similaridad\n");
           } else {
              out.message ("Similarity Matrix Clustering\n");
           }

           dialogSM = new DialogSimilarityMatrix (cfg,patterns,dm,this,out,help);
           dialogSM.setLocation((dim.width - dialogSM.getSize().width) / 2, (dim.height - dialogSM.getSize().height) / 2);
           dialogSM.show();
           break;
    }
  }

  // Clusters analysis

  public void showClustering ( ClusteringAlgorithm clustering,
                               Dataset             patterns,
                               String              title )
  {
    FrameClusters frameClusters = null;
    Dimension     dim = Toolkit.getDefaultToolkit().getScreenSize();

    if (clustering!=null) {
       frameClusters = new FrameClusters (cfg, clustering, patterns, title, out, help, miner, standalone);
       frameClusters.setLocation((dim.width - frameClusters.getSize().width) / 2, (dim.height - frameClusters.getSize().height) / 2);
       frameClusters.show();
    }
  }

  // HELP
  // ----

  void aboutNC ()
  {
    AboutBox  about = new AboutBox(this);
    Dimension d     = Toolkit.getDefaultToolkit().getScreenSize();

    about.setLocation((d.width - about.getSize().width) / 2, (d.height - about.getSize().height) / 2);
    about.show();
  }

  void memory ()
  {
    MemoryMonitorFrame frame = new MemoryMonitorFrame();
    Dimension          d     = Toolkit.getDefaultToolkit().getScreenSize();

    frame.setLocation((d.width - frame.getSize().width) / 2, (d.height - frame.getSize().height) / 2);
    frame.show();
  }

  void onlineHelp ()
  {
    help.help ( "nc/index.html" );
  }

}

// Event handlers
// --------------

class MainMenu_windowAdapter extends java.awt.event.WindowAdapter
{
  MainMenu adaptee;

  MainMenu_windowAdapter(MainMenu adaptee)
  {
    this.adaptee = adaptee;
  }

  public void windowClosing(WindowEvent e)
  {
    adaptee.exit();
  }
}


class MainMenu_menuItemExit_actionAdapter implements java.awt.event.ActionListener
{
  MainMenu adaptee;

  MainMenu_menuItemExit_actionAdapter(MainMenu adaptee)
  {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.exit();
  }
}

class MainMenu_menuItem_actionAdapter implements java.awt.event.ActionListener
{
  MainMenu adaptee;
  byte     option;

  MainMenu_menuItem_actionAdapter (MainMenu adaptee, byte option)
  {
    this.adaptee = adaptee;
    this.option  = option;
  }

  public void actionPerformed(ActionEvent e)
  {
    adaptee.actionPerformed(option);
  }
}
